@@ -78,7 +78,7 @@ struct _GstOggMap
   GstOggMapGranuleposToKeyGranuleFunc granulepos_to_key_granule_func;
 };
 
-static const GstOggMap mappers[];
+extern const GstOggMap mappers[];
 
 GstClockTime
 gst_ogg_stream_get_packet_start_time (GstOggStream * pad, ogg_packet * packet)
@@ -918,6 +918,9 @@ setup_fishead_mapper (GstOggStream * pad, ogg_packet * packet)
       GST_TIME_ARGS (pad->prestime));
 
   pad->is_skeleton = TRUE;
+  pad->is_sparse = TRUE;
+
+  pad->caps = gst_caps_new_simple ("none/none", NULL);
 
   return TRUE;
 }
@@ -1336,6 +1339,7 @@ setup_ogmtext_mapper (GstOggStream * pad, ogg_packet * packet)
   pad->n_header_packets = 1;
   pad->is_ogm = TRUE;
   pad->is_ogm_text = TRUE;
+  pad->is_sparse = TRUE;
 
   return TRUE;
 }
@@ -1493,6 +1497,7 @@ setup_cmml_mapper (GstOggStream * pad, ogg_packet * packet)
   GST_DEBUG ("blocksize1: %u", 1 << (data[0] & 0x0F));
 
   pad->caps = gst_caps_new_simple ("text/x-cmml", NULL);
+  pad->is_sparse = TRUE;
 
   return TRUE;
 }
@@ -1551,13 +1556,15 @@ setup_kate_mapper (GstOggStream * pad, ogg_packet * packet)
     pad->caps = gst_caps_new_simple ("application/x-kate", NULL);
   }
 
+  pad->is_sparse = TRUE;
+
   return TRUE;
 }
 
 
 /* *INDENT-OFF* */
 /* indent hates our freedoms */
-static const GstOggMap mappers[] = {
+const GstOggMap mappers[] = {
   {
     "\200theora", 7, 42,
     "video/x-theora",
