@@ -43,6 +43,7 @@
 #include "gst/gst-i18n-plugin.h"
 
 #include "pbutils.h"
+#include "pbutils-private.h"
 
 #include <string.h>
 
@@ -99,6 +100,7 @@ static const FormatInfo formats[] = {
   {"audio/x-amr-nb-sh", "Adaptive Multi Rate NarrowBand (AMR-NB)", 0},
   {"audio/x-amr-wb-sh", "Adaptive Multi Rate WideBand (AMR-WB)", 0},
   {"audio/x-au", "Sun .au", 0},
+  {"audio/x-celt", "Constrained Energy Lapped Transform (CELT)", 0},
   {"audio/x-cinepak", "Cinepak Audio", 0},
   {"audio/x-dpcm", "DPCM", 0},
   {"audio/x-dts", "DTS", 0},
@@ -156,6 +158,7 @@ static const FormatInfo formats[] = {
   {"video/x-cirrus-logic-accupak", "Cirrus Logipak AccuPak", 0},
   {"video/x-compressed-yuv", N_("CYUV Lossless"), 0},
   {"video/x-dirac", "Dirac", 0},
+  {"video/x-dnxhd", "Digital Nonlinear Extensible High Definition (DNxHD)", 0},
   {"video/x-dvd-subpicture", "DVD subpicture", 0},
   {"video/x-ffv", N_("FFMpeg v1"), 0},
   {"video/x-flash-screen", "Flash Screen Video", 0},
@@ -166,6 +169,7 @@ static const FormatInfo formats[] = {
   {"video/x-jpeg", "Motion JPEG", 0},
   /* { "video/x-jpeg-b", "", 0 }, does this actually exist? */
   {"video/x-loco", "LOCO Lossless", 0},
+  {"video/x-mimic", "MIMIC", 0},
   {"video/x-mjpeg", "Motion-JPEG", 0},
   {"video/x-mjpeg-b", "Motion-JPEG format B", 0},
   {"video/mpegts", "MPEG-2 Transport Stream", FLAG_CONTAINER},
@@ -777,23 +781,27 @@ gchar *
 gst_pb_utils_get_decoder_description (const GstCaps * caps)
 {
   gchar *str, *ret;
+  GstCaps *tmp;
 
   g_return_val_if_fail (caps != NULL, NULL);
   g_return_val_if_fail (GST_IS_CAPS (caps), NULL);
-  g_return_val_if_fail (gst_caps_is_fixed (caps), NULL);
+
+  tmp = copy_and_clean_caps (caps);
+
+  g_return_val_if_fail (gst_caps_is_fixed (tmp), NULL);
 
   /* special-case RTP caps */
-  if (caps_are_rtp_caps (caps, "video", &str)) {
+  if (caps_are_rtp_caps (tmp, "video", &str)) {
     ret = g_strdup_printf (_("%s video RTP depayloader"), str);
-  } else if (caps_are_rtp_caps (caps, "audio", &str)) {
+  } else if (caps_are_rtp_caps (tmp, "audio", &str)) {
     ret = g_strdup_printf (_("%s audio RTP depayloader"), str);
-  } else if (caps_are_rtp_caps (caps, "application", &str)) {
+  } else if (caps_are_rtp_caps (tmp, "application", &str)) {
     ret = g_strdup_printf (_("%s RTP depayloader"), str);
   } else {
     const FormatInfo *info;
 
-    str = gst_pb_utils_get_codec_description (caps);
-    info = find_format_info (caps);
+    str = gst_pb_utils_get_codec_description (tmp);
+    info = find_format_info (tmp);
     if (info != NULL && (info->flags & FLAG_CONTAINER) != 0) {
       ret = g_strdup_printf (_("%s demuxer"), str);
     } else {
@@ -802,6 +810,7 @@ gst_pb_utils_get_decoder_description (const GstCaps * caps)
   }
 
   g_free (str);
+  gst_caps_unref (tmp);
 
   return ret;
 }
@@ -825,23 +834,25 @@ gchar *
 gst_pb_utils_get_encoder_description (const GstCaps * caps)
 {
   gchar *str, *ret;
+  GstCaps *tmp;
 
   g_return_val_if_fail (caps != NULL, NULL);
   g_return_val_if_fail (GST_IS_CAPS (caps), NULL);
-  g_return_val_if_fail (gst_caps_is_fixed (caps), NULL);
+  tmp = copy_and_clean_caps (caps);
+  g_return_val_if_fail (gst_caps_is_fixed (tmp), NULL);
 
   /* special-case RTP caps */
-  if (caps_are_rtp_caps (caps, "video", &str)) {
+  if (caps_are_rtp_caps (tmp, "video", &str)) {
     ret = g_strdup_printf (_("%s video RTP payloader"), str);
-  } else if (caps_are_rtp_caps (caps, "audio", &str)) {
+  } else if (caps_are_rtp_caps (tmp, "audio", &str)) {
     ret = g_strdup_printf (_("%s audio RTP payloader"), str);
-  } else if (caps_are_rtp_caps (caps, "application", &str)) {
+  } else if (caps_are_rtp_caps (tmp, "application", &str)) {
     ret = g_strdup_printf (_("%s RTP payloader"), str);
   } else {
     const FormatInfo *info;
 
-    str = gst_pb_utils_get_codec_description (caps);
-    info = find_format_info (caps);
+    str = gst_pb_utils_get_codec_description (tmp);
+    info = find_format_info (tmp);
     if (info != NULL && (info->flags & FLAG_CONTAINER) != 0) {
       ret = g_strdup_printf (_("%s muxer"), str);
     } else {
@@ -850,6 +861,7 @@ gst_pb_utils_get_encoder_description (const GstCaps * caps)
   }
 
   g_free (str);
+  gst_caps_unref (tmp);
 
   return ret;
 }
@@ -939,17 +951,19 @@ gst_pb_utils_get_codec_description (const GstCaps * caps)
 {
   const FormatInfo *info;
   gchar *str, *comma;
+  GstCaps *tmp;
 
   g_return_val_if_fail (caps != NULL, NULL);
   g_return_val_if_fail (GST_IS_CAPS (caps), NULL);
-  g_return_val_if_fail (gst_caps_is_fixed (caps), NULL);
+  tmp = copy_and_clean_caps (caps);
+  g_return_val_if_fail (gst_caps_is_fixed (tmp), NULL);
 
-  info = find_format_info (caps);
+  info = find_format_info (tmp);
 
   if (info) {
-    str = format_info_get_desc (info, caps);
+    str = format_info_get_desc (info, tmp);
   } else {
-    str = gst_caps_to_string (caps);
+    str = gst_caps_to_string (tmp);
 
     /* cut off everything after the media type, if there is anything */
     if ((comma = strchr (str, ','))) {
@@ -961,6 +975,7 @@ gst_pb_utils_get_codec_description (const GstCaps * caps)
 
     GST_WARNING ("No description available for media type: %s", str);
   }
+  gst_caps_unref (tmp);
 
   return str;
 }
