@@ -39,9 +39,6 @@ static GstDiscovererAudioInfo
 static GstDiscovererVideoInfo
     * gst_discoverer_video_info_copy_int (GstDiscovererVideoInfo * ptr);
 
-static GstDiscovererSubtitleInfo
-    * gst_discoverer_subtitle_info_copy_int (GstDiscovererSubtitleInfo * ptr);
-
 /* Per-stream information */
 
 G_DEFINE_TYPE (GstDiscovererStreamInfo, gst_discoverer_stream_info,
@@ -113,11 +110,6 @@ gst_discoverer_info_copy_int (GstDiscovererStreamInfo * info,
     ret = (GstDiscovererStreamInfo *)
         gst_discoverer_video_info_copy_int ((GstDiscovererVideoInfo *) info);
 
-  } else if (ltyp == GST_TYPE_DISCOVERER_SUBTITLE_INFO) {
-    ret = (GstDiscovererStreamInfo *)
-        gst_discoverer_subtitle_info_copy_int ((GstDiscovererSubtitleInfo *)
-        info);
-
   } else
     ret = gst_discoverer_stream_info_new ();
 
@@ -206,24 +198,16 @@ gst_stream_container_info_copy_int (GstDiscovererContainerInfo * ptr,
 G_DEFINE_TYPE (GstDiscovererAudioInfo, gst_discoverer_audio_info,
     GST_TYPE_DISCOVERER_STREAM_INFO);
 
-static void
-gst_discoverer_audio_info_finalize (GstDiscovererAudioInfo * info)
-{
-  g_free (info->language);
-  gst_discoverer_stream_info_finalize ((GstDiscovererStreamInfo *) info);
-}
-
 static void
 gst_discoverer_audio_info_class_init (GstDiscovererAudioInfoClass * klass)
 {
-  klass->finalize =
-      (GstMiniObjectFinalizeFunction) gst_discoverer_audio_info_finalize;
+  /* Nothing to initialize */
 }
 
 static void
 gst_discoverer_audio_info_init (GstDiscovererAudioInfo * info)
 {
-  info->language = NULL;
+  /* Nothing to initialize */
 }
 
 static GstDiscovererAudioInfo *
@@ -245,50 +229,6 @@ gst_discoverer_audio_info_copy_int (GstDiscovererAudioInfo * ptr)
   ret->depth = ptr->depth;
   ret->bitrate = ptr->bitrate;
   ret->max_bitrate = ptr->max_bitrate;
-  ret->language = g_strdup (ptr->language);
-
-  return ret;
-}
-
-/* Subtitle information */
-G_DEFINE_TYPE (GstDiscovererSubtitleInfo, gst_discoverer_subtitle_info,
-    GST_TYPE_DISCOVERER_STREAM_INFO);
-
-static void
-gst_discoverer_subtitle_info_init (GstDiscovererSubtitleInfo * info)
-{
-  info->language = NULL;
-}
-
-static void
-gst_discoverer_subtitle_info_finalize (GstDiscovererSubtitleInfo * info)
-{
-  g_free (info->language);
-  gst_discoverer_stream_info_finalize ((GstDiscovererStreamInfo *) info);
-}
-
-static void
-gst_discoverer_subtitle_info_class_init (GstMiniObjectClass * klass)
-{
-  klass->finalize =
-      (GstMiniObjectFinalizeFunction) gst_discoverer_subtitle_info_finalize;
-}
-
-static GstDiscovererSubtitleInfo *
-gst_discoverer_subtitle_info_new (void)
-{
-  return (GstDiscovererSubtitleInfo *)
-      gst_mini_object_new (GST_TYPE_DISCOVERER_SUBTITLE_INFO);
-}
-
-static GstDiscovererSubtitleInfo *
-gst_discoverer_subtitle_info_copy_int (GstDiscovererSubtitleInfo * ptr)
-{
-  GstDiscovererSubtitleInfo *ret;
-
-  ret = gst_discoverer_subtitle_info_new ();
-
-  ret->language = g_strdup (ptr->language);
 
   return ret;
 }
@@ -374,13 +314,11 @@ GstDiscovererInfo *
 gst_discoverer_info_copy (GstDiscovererInfo * ptr)
 {
   GstDiscovererInfo *ret;
-  GHashTable *stream_map;
+  GHashTable *stream_map = g_hash_table_new (g_direct_hash, NULL);
   GList *tmp;
 
   g_return_val_if_fail (ptr != NULL, NULL);
 
-  stream_map = g_hash_table_new (g_direct_hash, NULL);
-
   ret = gst_discoverer_info_new ();
 
   ret->uri = g_strdup (ptr->uri);
@@ -439,16 +377,15 @@ gst_discoverer_stream_info_list_free (GList * infos)
 /**
  * gst_discoverer_info_get_streams:
  * @info: a #GstDiscovererInfo
- * @streamtype: a #GType derived from #GstDiscovererStreamInfo
+ * @streamtype: a #GType of #GstDiscovererStreamInfo
  *
  * Finds the #GstDiscovererStreamInfo contained in @info that match the
  * given @streamtype.
  *
- * Returns: (transfer full) (element-type Gst.DiscovererStreamInfo): A #GList of
- * matching #GstDiscovererStreamInfo. The caller should free it with
- * gst_discoverer_stream_info_list_free().
+ * Returns: A #GList of matching #GstDiscovererStreamInfo. The caller should
+ * free it with #gst_discoverer_stream_info_list_free.
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 GList *
 gst_discoverer_info_get_streams (GstDiscovererInfo * info, GType streamtype)
@@ -471,11 +408,10 @@ gst_discoverer_info_get_streams (GstDiscovererInfo * info, GType streamtype)
  *
  * Finds all the #GstDiscovererAudioInfo contained in @info
  *
- * Returns: (transfer full) (element-type Gst.DiscovererStreamInfo): A #GList of
- * matching #GstDiscovererStreamInfo. The caller should free it with
- * gst_discoverer_stream_info_list_free().
+ * Returns: A #GList of matching #GstDiscovererStreamInfo. The caller should
+ * free it with #gst_discoverer_stream_info_list_free.
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 GList *
 gst_discoverer_info_get_audio_streams (GstDiscovererInfo * info)
@@ -489,11 +425,10 @@ gst_discoverer_info_get_audio_streams (GstDiscovererInfo * info)
  *
  * Finds all the #GstDiscovererVideoInfo contained in @info
  *
- * Returns: (transfer full) (element-type Gst.DiscovererStreamInfo): A #GList of
- * matching #GstDiscovererStreamInfo. The caller should free it with
- * gst_discoverer_stream_info_list_free().
+ * Returns: A #GList of matching #GstDiscovererStreamInfo. The caller should
+ * free it with #gst_discoverer_stream_info_list_free.
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 GList *
 gst_discoverer_info_get_video_streams (GstDiscovererInfo * info)
@@ -501,36 +436,16 @@ gst_discoverer_info_get_video_streams (GstDiscovererInfo * info)
   return gst_discoverer_info_get_streams (info, GST_TYPE_DISCOVERER_VIDEO_INFO);
 }
 
-/**
- * gst_discoverer_info_get_subtitle_streams:
- * @info: a #GstDiscovererInfo
- *
- * Finds all the #GstDiscovererSubtitleInfo contained in @info
- *
- * Returns: (transfer full) (element-type Gst.DiscovererStreamInfo): A #GList of
- * matching #GstDiscovererStreamInfo. The caller should free it with
- * gst_discoverer_stream_info_list_free().
- *
- * Since: 0.10.36
- */
-GList *
-gst_discoverer_info_get_subtitle_streams (GstDiscovererInfo * info)
-{
-  return gst_discoverer_info_get_streams (info,
-      GST_TYPE_DISCOVERER_SUBTITLE_INFO);
-}
-
 /**
  * gst_discoverer_info_get_container_streams:
  * @info: a #GstDiscovererInfo
  *
  * Finds all the #GstDiscovererContainerInfo contained in @info
  *
- * Returns: (transfer full) (element-type Gst.DiscovererStreamInfo): A #GList of
- * matching #GstDiscovererStreamInfo. The caller should free it with
- * gst_discoverer_stream_info_list_free().
+ * Returns: A #GList of matching #GstDiscovererStreamInfo. The caller should
+ * free it with #g_list_free.
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 GList *
 gst_discoverer_info_get_container_streams (GstDiscovererInfo * info)
@@ -546,7 +461,7 @@ gst_discoverer_info_get_container_streams (GstDiscovererInfo * info)
  * Returns: a human readable name for the stream type of the given @info (ex : "audio",
  * "container",...).
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 const gchar *
 gst_discoverer_stream_info_get_stream_type_nick (GstDiscovererStreamInfo * info)
@@ -556,35 +471,30 @@ gst_discoverer_stream_info_get_stream_type_nick (GstDiscovererStreamInfo * info)
   if (GST_IS_DISCOVERER_AUDIO_INFO (info))
     return "audio";
   if (GST_IS_DISCOVERER_VIDEO_INFO (info)) {
-    if (gst_discoverer_video_info_is_image ((GstDiscovererVideoInfo *)
-            info))
+    if (gst_discoverer_video_info_get_is_image (info))
       return "video(image)";
     else
       return "video";
   }
-  if (GST_IS_DISCOVERER_SUBTITLE_INFO (info))
-    return "subtitles";
   return "unknown";
 }
 
 /* ACCESSORS */
 
-
-#define GENERIC_ACCESSOR_CODE(parent, parenttype, parentgtype, fieldname, type, failval) \
-  type parent##_get_##fieldname(const parenttype info) {			\
+#define GENERIC_ACCESSOR_CODE(parent, casttype, parenttype, parentgtype, fieldname, type, failval) \
+  type parent##_get_##fieldname(parenttype info) {			\
     g_return_val_if_fail(G_TYPE_CHECK_INSTANCE_TYPE((info), parentgtype), failval); \
-    return (info)->fieldname;				\
+    return ((casttype)info)->fieldname;				\
   }
 
 /**
  * gst_discoverer_stream_info_get_previous:
  * @info: a #GstDiscovererStreamInfo
  *
- * Returns: (transfer full): the previous #GstDiscovererStreamInfo in a chain.
- * %NULL for starting points. Unref with #gst_discoverer_stream_info_unref
- * after usage.
+ * Returns: the previous #GstDiscovererStreamInfo in a chain. %NULL for starting
+ * points. Unref with #gst_discoverer_stream_info_unref after usage.
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 GstDiscovererStreamInfo *
 gst_discoverer_stream_info_get_previous (GstDiscovererStreamInfo * info)
@@ -600,11 +510,10 @@ gst_discoverer_stream_info_get_previous (GstDiscovererStreamInfo * info)
  * gst_discoverer_stream_info_get_next:
  * @info: a #GstDiscovererStreamInfo
  *
- * Returns: (transfer full): the next #GstDiscovererStreamInfo in a chain. %NULL
- * for final streams.
+ * Returns: the next #GstDiscovererStreamInfo in a chain. %NULL for final streams.
  * Unref with #gst_discoverer_stream_info_unref after usage.
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 GstDiscovererStreamInfo *
 gst_discoverer_stream_info_get_next (GstDiscovererStreamInfo * info)
@@ -621,29 +530,26 @@ gst_discoverer_stream_info_get_next (GstDiscovererStreamInfo * info)
  * gst_discoverer_stream_info_get_caps:
  * @info: a #GstDiscovererStreamInfo
  *
- * Returns: (transfer full): the #GstCaps of the stream. Unref with
- * #gst_caps_unref after usage.
+ * Returns: the #GstCaps of the stream. Unref with #gst_caps_unref after usage.
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 GstCaps *
 gst_discoverer_stream_info_get_caps (GstDiscovererStreamInfo * info)
 {
   g_return_val_if_fail (GST_IS_DISCOVERER_STREAM_INFO (info), NULL);
 
-  if (info->caps)
-    return gst_caps_ref (info->caps);
-  return NULL;
+  return gst_caps_ref (info->caps);
 }
 
 /**
  * gst_discoverer_stream_info_get_tags:
  * @info: a #GstDiscovererStreamInfo
  *
- * Returns: (transfer none): the tags contained in this stream. If you wish to
- * use the tags after the life-time of @info you will need to copy them.
+ * Returns: the tags contained in this stream. If you wish to use the tags after
+ * the life-time of @info you will need to copy them.
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 
 const GstTagList *
@@ -658,11 +564,11 @@ gst_discoverer_stream_info_get_tags (GstDiscovererStreamInfo * info)
  * gst_discoverer_stream_info_get_misc:
  * @info: a #GstDiscovererStreamInfo
  *
- * Returns: (transfer none): additional information regarding the stream (for
- * example codec version, profile, etc..). If you wish to use the #GstStructure
- * after the life-time of @info you will need to copy it.
+ * Returns: additional information regarding the stream (for example codec version,
+ * profile, etc..). If you wish to use the #GstStructure after the life-time of
+ * @info you will need to copy it.
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 const GstStructure *
 gst_discoverer_stream_info_get_misc (GstDiscovererStreamInfo * info)
@@ -678,21 +584,21 @@ gst_discoverer_stream_info_get_misc (GstDiscovererStreamInfo * info)
  * gst_discoverer_container_info_get_streams:
  * @info: a #GstDiscovererStreamInfo
  *
- * Returns: (transfer full) (element-type Gst.DiscovererStreamInfo): the list of
- * #GstDiscovererStreamInfo this container stream offers.
- * Free with gst_discoverer_stream_info_list_free() after usage.
+ * Returns: the list of #GstDiscovererStreamInfo this container stream offers.
+ * Free with #gst_discoverer_stream_info_list_free after usage.
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 
 GList *
-gst_discoverer_container_info_get_streams (GstDiscovererContainerInfo * info)
+gst_discoverer_container_info_get_streams (GstDiscovererStreamInfo * info)
 {
   GList *res = NULL, *tmp;
 
   g_return_val_if_fail (GST_IS_DISCOVERER_CONTAINER_INFO (info), NULL);
 
-  for (tmp = info->streams; tmp; tmp = tmp->next)
+  for (tmp = GST_DISCOVERER_CONTAINER_INFO (info)->streams; tmp;
+      tmp = tmp->next)
     res =
         g_list_append (res,
         gst_discoverer_stream_info_ref ((GstDiscovererStreamInfo *) tmp->data));
@@ -704,236 +610,201 @@ gst_discoverer_container_info_get_streams (GstDiscovererContainerInfo * info)
 
 #define AUDIO_INFO_ACCESSOR_CODE(fieldname, type, failval)		\
   GENERIC_ACCESSOR_CODE(gst_discoverer_audio_info, GstDiscovererAudioInfo*, \
+			GstDiscovererStreamInfo*,			\
 			GST_TYPE_DISCOVERER_AUDIO_INFO,		\
 			fieldname, type, failval)
 
 /**
  * gst_discoverer_audio_info_get_channels:
- * @info: a #GstDiscovererAudioInfo
+ * @info: a #GstDiscovererStreamInfo
  *
  * Returns: the number of channels in the stream.
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 
 AUDIO_INFO_ACCESSOR_CODE (channels, guint, 0);
 
 /**
  * gst_discoverer_audio_info_get_sample_rate:
- * @info: a #GstDiscovererAudioInfo
+ * @info: a #GstDiscovererStreamInfo
  *
  * Returns: the sample rate of the stream in Hertz.
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 
 AUDIO_INFO_ACCESSOR_CODE (sample_rate, guint, 0);
 
 /**
  * gst_discoverer_audio_info_get_depth:
- * @info: a #GstDiscovererAudioInfo
+ * @info: a #GstDiscovererStreamInfo
  *
  * Returns: the number of bits used per sample in each channel.
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 
 AUDIO_INFO_ACCESSOR_CODE (depth, guint, 0);
 
 /**
  * gst_discoverer_audio_info_get_bitrate:
- * @info: a #GstDiscovererAudioInfo
+ * @info: a #GstDiscovererStreamInfo
  *
  * Returns: the average or nominal bitrate of the stream in bits/second.
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 
 AUDIO_INFO_ACCESSOR_CODE (bitrate, guint, 0);
 
 /**
  * gst_discoverer_audio_info_get_max_bitrate:
- * @info: a #GstDiscovererAudioInfo
+ * @info: a #GstDiscovererStreamInfo
  *
  * Returns: the maximum bitrate of the stream in bits/second.
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 
 AUDIO_INFO_ACCESSOR_CODE (max_bitrate, guint, 0);
 
-/**
- * gst_discoverer_audio_info_get_language:
- * @info: a #GstDiscovererAudioInfo
- *
- * Returns: the language of the stream, or NULL if unknown.
- *
- * Since: 0.10.36
- */
-
-AUDIO_INFO_ACCESSOR_CODE (language, const gchar *, NULL);
-
 /* GstDiscovererVideoInfo */
 
 #define VIDEO_INFO_ACCESSOR_CODE(fieldname, type, failval)		\
   GENERIC_ACCESSOR_CODE(gst_discoverer_video_info, GstDiscovererVideoInfo*, \
+			GstDiscovererStreamInfo*,			\
 			GST_TYPE_DISCOVERER_VIDEO_INFO,			\
 			fieldname, type, failval)
 
 /**
  * gst_discoverer_video_info_get_width:
- * @info: a #GstDiscovererVideoInfo
+ * @info: a #GstDiscovererStreamInfo
  *
  * Returns: the width of the video stream in pixels.
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 
 VIDEO_INFO_ACCESSOR_CODE (width, guint, 0);
 
 /**
  * gst_discoverer_video_info_get_height:
- * @info: a #GstDiscovererVideoInfo
+ * @info: a #GstDiscovererStreamInfo
  *
  * Returns: the height of the video stream in pixels.
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 
 VIDEO_INFO_ACCESSOR_CODE (height, guint, 0);
 
 /**
  * gst_discoverer_video_info_get_depth:
- * @info: a #GstDiscovererVideoInfo
+ * @info: a #GstDiscovererStreamInfo
  *
- * Returns: the depth in bits of the video stream.
+ * Returns: the depth in bits of the video stream (only relevant for
+ * video streams).
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 
 VIDEO_INFO_ACCESSOR_CODE (depth, guint, 0);
 
 /**
  * gst_discoverer_video_info_get_framerate_num:
- * @info: a #GstDiscovererVideoInfo
+ * @info: a #GstDiscovererStreamInfo
  *
  * Returns: the framerate of the video stream (numerator).
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 
 VIDEO_INFO_ACCESSOR_CODE (framerate_num, guint, 0);
 
 /**
  * gst_discoverer_video_info_get_framerate_denom:
- * @info: a #GstDiscovererVideoInfo
+ * @info: a #GstDiscovererStreamInfo
  *
  * Returns: the framerate of the video stream (denominator).
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 
 VIDEO_INFO_ACCESSOR_CODE (framerate_denom, guint, 0);
 
 /**
  * gst_discoverer_video_info_get_par_num:
- * @info: a #GstDiscovererVideoInfo
+ * @info: a #GstDiscovererStreamInfo
  *
  * Returns: the Pixel Aspect Ratio (PAR) of the video stream (numerator).
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 
 VIDEO_INFO_ACCESSOR_CODE (par_num, guint, 0);
 
 /**
  * gst_discoverer_video_info_get_par_denom:
- * @info: a #GstDiscovererVideoInfo
+ * @info: a #GstDiscovererStreamInfo
  *
  * Returns: the Pixel Aspect Ratio (PAR) of the video stream (denominator).
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 
 VIDEO_INFO_ACCESSOR_CODE (par_denom, guint, 0);
 
 /**
- * gst_discoverer_video_info_is_interlaced:
- * @info: a #GstDiscovererVideoInfo
+ * gst_discoverer_video_info_get_interlaced:
+ * @info: a #GstDiscovererStreamInfo
  *
  * Returns: %TRUE if the stream is interlaced, else %FALSE.
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
-gboolean
-gst_discoverer_video_info_is_interlaced (const GstDiscovererVideoInfo * info)
-{
-  g_return_val_if_fail (GST_IS_DISCOVERER_VIDEO_INFO (info), FALSE);
 
-  return info->interlaced;
-}
+VIDEO_INFO_ACCESSOR_CODE (interlaced, gboolean, FALSE);
 
 /**
  * gst_discoverer_video_info_get_bitrate:
- * @info: a #GstDiscovererVideoInfo
+ * @info: a #GstDiscovererStreamInfo
  *
  * Returns: the average or nominal bitrate of the video stream in bits/second.
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 
 VIDEO_INFO_ACCESSOR_CODE (bitrate, guint, 0);
 
 /**
  * gst_discoverer_video_info_get_max_bitrate:
- * @info: a #GstDiscovererVideoInfo
+ * @info: a #GstDiscovererStreamInfo
  *
  * Returns: the maximum bitrate of the video stream in bits/second.
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 
 VIDEO_INFO_ACCESSOR_CODE (max_bitrate, guint, 0);
 
 /**
- * gst_discoverer_video_info_is_image:
- * @info: a #GstDiscovererVideoInfo
+ * gst_discoverer_video_info_get_is_image:
+ * @info: a #GstDiscovererStreamInfo
  *
  * Returns: #TRUE if the video stream corresponds to an image (i.e. only contains
  * one frame).
  *
- * Since: 0.10.31
- */
-gboolean
-gst_discoverer_video_info_is_image (const GstDiscovererVideoInfo * info)
-{
-  g_return_val_if_fail (GST_IS_DISCOVERER_VIDEO_INFO (info), FALSE);
-
-  return info->is_image;
-}
-
-/* GstDiscovererSubtitleInfo */
-
-#define SUBTITLE_INFO_ACCESSOR_CODE(fieldname, type, failval)                     \
-  GENERIC_ACCESSOR_CODE(gst_discoverer_subtitle_info, GstDiscovererSubtitleInfo*, \
-			GST_TYPE_DISCOVERER_SUBTITLE_INFO,                        \
-			fieldname, type, failval)
-
-/**
- * gst_discoverer_subtitle_info_get_language:
- * @info: a #GstDiscovererSubtitleInfo
- *
- * Returns: the language of the stream, or NULL if unknown.
- *
- * Since: 0.10.36
+ * Since 0.10.31
  */
 
-SUBTITLE_INFO_ACCESSOR_CODE (language, const gchar *, NULL);
+VIDEO_INFO_ACCESSOR_CODE (is_image, gboolean, FALSE);
 
 /* GstDiscovererInfo */
 
 #define DISCOVERER_INFO_ACCESSOR_CODE(fieldname, type, failval)		\
   GENERIC_ACCESSOR_CODE(gst_discoverer_info, GstDiscovererInfo*,	\
+			GstDiscovererInfo*,				\
 			GST_TYPE_DISCOVERER_INFO,			\
 			fieldname, type, failval)
 
@@ -941,10 +812,10 @@ SUBTITLE_INFO_ACCESSOR_CODE (language, const gchar *, NULL);
  * gst_discoverer_info_get_uri:
  * @info: a #GstDiscovererInfo
  *
- * Returns: (transfer none): the URI to which this information corresponds to.
- * Copy it if you wish to use it after the life-time of @info.
+ * Returns: the URI to which this information corresponds to. Copy it if you
+ * wish to use it after the life-time of @info.
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 
 DISCOVERER_INFO_ACCESSOR_CODE (uri, const gchar *, NULL);
@@ -955,7 +826,7 @@ DISCOVERER_INFO_ACCESSOR_CODE (uri, const gchar *, NULL);
  *
  * Returns: the result of the discovery as a #GstDiscovererResult.
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 
 DISCOVERER_INFO_ACCESSOR_CODE (result, GstDiscovererResult, GST_DISCOVERER_OK);
@@ -964,12 +835,11 @@ DISCOVERER_INFO_ACCESSOR_CODE (result, GstDiscovererResult, GST_DISCOVERER_OK);
  * gst_discoverer_info_get_stream_info:
  * @info: a #GstDiscovererInfo
  *
- * Returns: (transfer full): the structure (or topology) of the URI as a
- * #GstDiscovererStreamInfo.
+ * Returns: the structure (or topology) of the URI as a #GstDiscovererStreamInfo.
  * This structure can be traversed to see the original hierarchy. Unref with
- * gst_discoverer_stream_info_unref() after usage.
+ * #gst_discoverer_stream_info_unref after usage.
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 
 GstDiscovererStreamInfo *
@@ -986,11 +856,10 @@ gst_discoverer_info_get_stream_info (GstDiscovererInfo * info)
  * gst_discoverer_info_get_stream_list:
  * @info: a #GstDiscovererInfo
  *
- * Returns: (transfer full) (element-type Gst.DiscovererStreamInfo): the list of
- * all streams contained in the #info. Free after usage
- * with gst_discoverer_stream_info_list_free().
+ * Returns: the list of all streams contained in the #info. Free after usage
+ * with #gst_discoverer_stream_info_list_free.
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 GList *
 gst_discoverer_info_get_stream_list (GstDiscovererInfo * info)
@@ -1013,31 +882,20 @@ gst_discoverer_info_get_stream_list (GstDiscovererInfo * info)
  *
  * Returns: the duration of the URI in #GstClockTime (nanoseconds).
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 
 DISCOVERER_INFO_ACCESSOR_CODE (duration, GstClockTime, GST_CLOCK_TIME_NONE);
 
-/**
- * gst_discoverer_info_get_seekable:
- * @info: a #GstDiscovererInfo
- *
- * Returns: the whether the URI is seekable.
- *
- * Since: 0.10.32
- */
-
-DISCOVERER_INFO_ACCESSOR_CODE (seekable, gboolean, FALSE);
-
 /**
  * gst_discoverer_info_get_misc:
  * @info: a #GstDiscovererInfo
  *
- * Returns: (transfer none): Miscellaneous information stored as a #GstStructure
- * (for example: information about missing plugins). If you wish to use the
- * #GstStructure after the life-time of @info, you will need to copy it.
+ * Returns: Miscellaneous information stored as a #GstStructure (for example:
+ * information about missing plugins). If you wish to use the #GstStructure
+ * after the life-time of @info, you will need to copy it.
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 
 DISCOVERER_INFO_ACCESSOR_CODE (misc, const GstStructure *, NULL);
@@ -1046,50 +904,10 @@ DISCOVERER_INFO_ACCESSOR_CODE (misc, const GstStructure *, NULL);
  * gst_discoverer_info_get_tags:
  * @info: a #GstDiscovererInfo
  *
- * Returns: (transfer none): all tags contained in the URI. If you wish to use
- * the tags after the life-time of @info, you will need to copy them.
+ * Returns: all tags contained in the %URI. If you wish to use the tags after
+ * the life-time of @info, you will need to copy them.
  *
- * Since: 0.10.31
+ * Since 0.10.31
  */
 
 DISCOVERER_INFO_ACCESSOR_CODE (tags, const GstTagList *, NULL);
-
-/**
- * gst_discoverer_info_ref:
- * @info: a #GstDiscovererInfo
- *
- * Increments the reference count of @info.
- *
- * Returns: the same #GstDiscovererInfo object
- *
- * Since: 0.10.31
- */
-
-/**
- * gst_discoverer_info_unref:
- * @info: a #GstDiscovererInfo
- *
- * Decrements the reference count of @info.
- *
- * Since: 0.10.31
- */
-
-/**
- * gst_discoverer_stream_info_ref:
- * @info: a #GstDiscovererStreamInfo
- *
- * Increments the reference count of @info.
- *
- * Returns: the same #GstDiscovererStreamInfo object
- *
- * Since: 0.10.31
- */
-
-/**
- * gst_discoverer_stream_info_unref:
- * @info: a #GstDiscovererStreamInfo
- *
- * Decrements the reference count of @info.
- *
- * Since: 0.10.31
- */
