@@ -36,20 +36,19 @@
  * asks for the discovery to begin (through gst_discoverer_start()).
  *
  * All the information is returned in a #GstDiscovererInfo structure.
- *
- * Since: 0.10.31
+ * 
+ * Since 0.10.31
  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#include <gst/video/video.h>
 #include "pbutils.h"
 #include "pbutils-marshal.h"
 #include "pbutils-private.h"
 
-#include "gst/glib-compat-private.h"
-
 GST_DEBUG_CATEGORY_STATIC (discoverer_debug);
 #define GST_CAT_DEFAULT discoverer_debug
 
@@ -106,15 +105,6 @@ struct _GstDiscovererPrivate
   GMainContext *ctx;
   guint sourceid;
   guint timeoutid;
-
-  /* reusable queries */
-  GstQuery *seeking_query;
-
-  /* Handler ids for various callbacks */
-  gulong pad_added_id;
-  gulong pad_remove_id;
-  gulong element_added_id;
-  gulong bus_cb_id;
 };
 
 #define DISCO_LOCK(dc) g_mutex_lock (dc->priv->lock);
@@ -225,7 +215,7 @@ gst_discoverer_class_init (GstDiscovererClass * klass)
    * @discoverer: the #GstDiscoverer
    * @info: the results #GstDiscovererInfo
    * @error: (type GLib.Error): #GError, which will be non-NULL if an error
-   *                            occurred during discovery
+   *                            occured during discovery
    *
    * Will be emitted when all information on a URI could be discovered.
    */
@@ -252,7 +242,6 @@ static void
 gst_discoverer_init (GstDiscoverer * dc)
 {
   GstElement *tmp;
-  GstFormat format = GST_FORMAT_TIME;
 
   dc->priv = G_TYPE_INSTANCE_GET_PRIVATE (dc, GST_TYPE_DISCOVERER,
       GstDiscovererPrivate);
@@ -274,34 +263,27 @@ gst_discoverer_init (GstDiscoverer * dc)
   GST_LOG_OBJECT (dc, "Adding uridecodebin to pipeline");
   gst_bin_add (dc->priv->pipeline, dc->priv->uridecodebin);
 
-  dc->priv->pad_added_id =
-      g_signal_connect_object (dc->priv->uridecodebin, "pad-added",
-      G_CALLBACK (uridecodebin_pad_added_cb), dc, 0);
-  dc->priv->pad_remove_id =
-      g_signal_connect_object (dc->priv->uridecodebin, "pad-removed",
-      G_CALLBACK (uridecodebin_pad_removed_cb), dc, 0);
+  g_signal_connect (dc->priv->uridecodebin, "pad-added",
+      G_CALLBACK (uridecodebin_pad_added_cb), dc);
+  g_signal_connect (dc->priv->uridecodebin, "pad-removed",
+      G_CALLBACK (uridecodebin_pad_removed_cb), dc);
 
   GST_LOG_OBJECT (dc, "Getting pipeline bus");
   dc->priv->bus = gst_pipeline_get_bus ((GstPipeline *) dc->priv->pipeline);
 
-  dc->priv->bus_cb_id =
-      g_signal_connect_object (dc->priv->bus, "message",
-      G_CALLBACK (discoverer_bus_cb), dc, 0);
+  g_signal_connect (dc->priv->bus, "message", G_CALLBACK (discoverer_bus_cb),
+      dc);
 
   GST_DEBUG_OBJECT (dc, "Done initializing Discoverer");
 
   /* This is ugly. We get the GType of decodebin2 so we can quickly detect
    * when a decodebin2 is added to uridecodebin so we can set the
    * post-stream-topology setting to TRUE */
-  dc->priv->element_added_id =
-      g_signal_connect_object (dc->priv->uridecodebin, "element-added",
-      G_CALLBACK (uridecodebin_element_added_cb), dc, 0);
+  g_signal_connect (dc->priv->uridecodebin, "element-added",
+      G_CALLBACK (uridecodebin_element_added_cb), dc);
   tmp = gst_element_factory_make ("decodebin2", NULL);
   dc->priv->decodebin2_type = G_OBJECT_TYPE (tmp);
   gst_object_unref (tmp);
-
-  /* create queries */
-  dc->priv->seeking_query = gst_query_new_seeking (format);
 }
 
 static void
@@ -315,16 +297,9 @@ discoverer_reset (GstDiscoverer * dc)
     dc->priv->pending_uris = NULL;
   }
 
-  if (dc->priv->pipeline)
-    gst_element_set_state ((GstElement *) dc->priv->pipeline, GST_STATE_NULL);
+  gst_element_set_state ((GstElement *) dc->priv->pipeline, GST_STATE_NULL);
 }
 
-#define DISCONNECT_SIGNAL(o,i) G_STMT_START{           \
-  if ((i) && g_signal_handler_is_connected ((o), (i))) \
-    g_signal_handler_disconnect ((o), (i));            \
-  (i) = 0;                                             \
-}G_STMT_END
-
 static void
 gst_discoverer_dispose (GObject * obj)
 {
@@ -335,34 +310,18 @@ gst_discoverer_dispose (GObject * obj)
   discoverer_reset (dc);
 
   if (G_LIKELY (dc->priv->pipeline)) {
-    /* Workaround for bug #118536 */
-    DISCONNECT_SIGNAL (dc->priv->uridecodebin, dc->priv->pad_added_id);
-    DISCONNECT_SIGNAL (dc->priv->uridecodebin, dc->priv->pad_remove_id);
-    DISCONNECT_SIGNAL (dc->priv->uridecodebin, dc->priv->element_added_id);
-    DISCONNECT_SIGNAL (dc->priv->bus, dc->priv->bus_cb_id);
-
     /* pipeline was set to NULL in _reset */
     gst_object_unref (dc->priv->pipeline);
     gst_object_unref (dc->priv->bus);
-
     dc->priv->pipeline = NULL;
     dc->priv->uridecodebin = NULL;
     dc->priv->bus = NULL;
   }
 
-  gst_discoverer_stop (dc);
-
   if (dc->priv->lock) {
     g_mutex_free (dc->priv->lock);
     dc->priv->lock = NULL;
   }
-
-  if (dc->priv->seeking_query) {
-    gst_query_unref (dc->priv->seeking_query);
-    dc->priv->seeking_query = NULL;
-  }
-
-  G_OBJECT_CLASS (gst_discoverer_parent_class)->dispose (obj);
 }
 
 static void
@@ -414,55 +373,24 @@ static gboolean
 _event_probe (GstPad * pad, GstEvent * event, PrivateStream * ps)
 {
   if (GST_EVENT_TYPE (event) == GST_EVENT_TAG) {
-    GstTagList *tl = NULL, *tmp;
+    GstTagList *tl = NULL;
 
     gst_event_parse_tag (event, &tl);
     GST_DEBUG_OBJECT (pad, "tags %" GST_PTR_FORMAT, tl);
     DISCO_LOCK (ps->dc);
-    /* If preroll is complete, drop these tags - the collected information is
-     * possibly already being processed and adding more tags would be racy */
-    if (G_LIKELY (ps->dc->priv->processing)) {
-      GST_DEBUG_OBJECT (pad, "private stream %p old tags %" GST_PTR_FORMAT, ps,
-          ps->tags);
-      tmp = gst_tag_list_merge (ps->tags, tl, GST_TAG_MERGE_APPEND);
-      if (ps->tags)
-        gst_tag_list_free (ps->tags);
-      ps->tags = tmp;
-      GST_DEBUG_OBJECT (pad, "private stream %p new tags %" GST_PTR_FORMAT, ps,
-          tmp);
-    } else
-      GST_DEBUG_OBJECT (pad, "Dropping tags since preroll is done");
+    ps->tags = gst_tag_list_merge (ps->tags, tl, GST_TAG_MERGE_APPEND);
     DISCO_UNLOCK (ps->dc);
   }
 
   return TRUE;
 }
 
-static GstStaticCaps subtitle_caps = GST_STATIC_CAPS ("text/plain; "
-    "text/x-pango-markup; subpicture/x-pgs; subpicture/x-dvb; "
-    "application/x-subtitle-unknown; application/x-ssa; application/x-ass; "
-    "subtitle/x-kate; application/x-kate; video/x-dvd-subpicture");
-
-static gboolean
-is_subtitle_caps (const GstCaps * caps)
-{
-  GstCaps *subs_caps;
-  gboolean ret;
-
-  subs_caps = gst_static_caps_get (&subtitle_caps);
-  ret = gst_caps_can_intersect (caps, subs_caps);
-  gst_caps_unref (subs_caps);
-
-  return ret;
-}
-
 static void
 uridecodebin_pad_added_cb (GstElement * uridecodebin, GstPad * pad,
     GstDiscoverer * dc)
 {
   PrivateStream *ps;
   GstPad *sinkpad = NULL;
-  GstCaps *caps;
 
   GST_DEBUG_OBJECT (dc, "pad %s:%s", GST_DEBUG_PAD_NAME (pad));
 
@@ -479,16 +407,6 @@ uridecodebin_pad_added_cb (GstElement * uridecodebin, GstPad * pad,
   g_object_set (ps->sink, "silent", TRUE, NULL);
   g_object_set (ps->queue, "max-size-buffers", 1, "silent", TRUE, NULL);
 
-  caps = gst_pad_get_caps_reffed (pad);
-
-  if (is_subtitle_caps (caps)) {
-    /* Subtitle streams are sparse and may not provide any information - don't
-     * wait for data to preroll */
-    g_object_set (ps->sink, "async", FALSE, NULL);
-  }
-
-  gst_caps_unref (caps);
-
   gst_bin_add_many (dc->priv->pipeline, ps->queue, ps->sink, NULL);
 
   if (!gst_element_link_pads_full (ps->queue, "src", ps->sink, "sink",
@@ -590,40 +508,20 @@ collect_stream_information (GstDiscoverer * dc, PrivateStream * ps, guint idx)
 
   /* Get caps */
   caps = gst_pad_get_negotiated_caps (ps->pad);
-  if (!caps) {
-    GST_WARNING ("Couldn't get negotiated caps from %s:%s",
-        GST_DEBUG_PAD_NAME (ps->pad));
-    caps = gst_pad_get_caps (ps->pad);
-  }
   if (caps) {
     GST_DEBUG ("Got caps %" GST_PTR_FORMAT, caps);
     gst_structure_id_set (st, _CAPS_QUARK, GST_TYPE_CAPS, caps, NULL);
 
     gst_caps_unref (caps);
-  }
+  } else
+    GST_WARNING ("Couldn't get negotiated caps from %s:%s",
+        GST_DEBUG_PAD_NAME (ps->pad));
   if (ps->tags)
-    gst_structure_id_set (st, _TAGS_QUARK, GST_TYPE_TAG_LIST, ps->tags, NULL);
+    gst_structure_id_set (st, _TAGS_QUARK, GST_TYPE_STRUCTURE, ps->tags, NULL);
 
   return st;
 }
 
-/* takes ownership of new_tags, may replace *taglist with a new one */
-static void
-gst_discoverer_merge_and_replace_tags (GstTagList ** taglist,
-    GstTagList * new_tags)
-{
-  if (new_tags == NULL)
-    return;
-
-  if (*taglist == NULL) {
-    *taglist = new_tags;
-    return;
-  }
-
-  gst_tag_list_insert (*taglist, new_tags, GST_TAG_MERGE_REPLACE);
-  gst_tag_list_free (new_tags);
-}
-
 /* Parses a set of caps and tags in st and populates a GstDiscovererStreamInfo
  * structure (parent, if !NULL, otherwise it allocates one)
  */
@@ -632,16 +530,16 @@ collect_information (GstDiscoverer * dc, const GstStructure * st,
     GstDiscovererStreamInfo * parent)
 {
   GstCaps *caps;
-  GstStructure *caps_st;
-  GstTagList *tags_st;
+  GstStructure *caps_st, *tags_st;
   const gchar *name;
   int tmp, tmp2;
   guint utmp;
+  gboolean btmp;
 
   if (!st || !gst_structure_id_has_field (st, _CAPS_QUARK)) {
     GST_WARNING ("Couldn't find caps !");
     if (parent)
-      return gst_discoverer_stream_info_ref (parent);
+      return parent;
     else
       return (GstDiscovererStreamInfo *)
           gst_mini_object_new (GST_TYPE_DISCOVERER_STREAM_INFO);
@@ -655,11 +553,11 @@ collect_information (GstDiscoverer * dc, const GstStructure * st,
     GstDiscovererAudioInfo *info;
 
     if (parent)
-      info = (GstDiscovererAudioInfo *) gst_discoverer_stream_info_ref (parent);
+      info = (GstDiscovererAudioInfo *) parent;
     else {
       info = (GstDiscovererAudioInfo *)
           gst_mini_object_new (GST_TYPE_DISCOVERER_AUDIO_INFO);
-      info->parent.caps = gst_caps_ref (caps);
+      info->parent.caps = caps;
     }
 
     if (gst_structure_get_int (caps_st, "rate", &tmp))
@@ -672,43 +570,37 @@ collect_information (GstDiscoverer * dc, const GstStructure * st,
       info->depth = (guint) tmp;
 
     if (gst_structure_id_has_field (st, _TAGS_QUARK)) {
-      gst_structure_id_get (st, _TAGS_QUARK, GST_TYPE_TAG_LIST, &tags_st, NULL);
-      if (gst_structure_get_uint (tags_st, GST_TAG_BITRATE, &utmp) ||
-          gst_structure_get_uint (tags_st, GST_TAG_NOMINAL_BITRATE, &utmp))
+      gst_structure_id_get (st, _TAGS_QUARK,
+          GST_TYPE_STRUCTURE, &tags_st, NULL);
+      if (gst_structure_get_uint (tags_st, GST_TAG_BITRATE, &utmp))
         info->bitrate = utmp;
 
       if (gst_structure_get_uint (tags_st, GST_TAG_MAXIMUM_BITRATE, &utmp))
         info->max_bitrate = utmp;
 
       /* FIXME: Is it worth it to remove the tags we've parsed? */
-      gst_discoverer_merge_and_replace_tags (&info->parent.tags, tags_st);
+      info->parent.tags = gst_tag_list_merge (info->parent.tags,
+          (GstTagList *) tags_st, GST_TAG_MERGE_REPLACE);
+
+      gst_structure_free (tags_st);
     }
 
-    if (!info->language && ((GstDiscovererStreamInfo *) info)->tags) {
-      gchar *language;
-      if (gst_tag_list_get_string (((GstDiscovererStreamInfo *) info)->tags,
-              GST_TAG_LANGUAGE_CODE, &language)) {
-        info->language = language;
-      }
-    }
-
-    gst_caps_unref (caps);
     return (GstDiscovererStreamInfo *) info;
 
   } else if (g_str_has_prefix (name, "video/") ||
       g_str_has_prefix (name, "image/")) {
     GstDiscovererVideoInfo *info;
+    GstVideoFormat format;
 
     if (parent)
-      info = (GstDiscovererVideoInfo *) gst_discoverer_stream_info_ref (parent);
+      info = (GstDiscovererVideoInfo *) parent;
     else {
       info = (GstDiscovererVideoInfo *)
           gst_mini_object_new (GST_TYPE_DISCOVERER_VIDEO_INFO);
-      info->parent.caps = gst_caps_ref (caps);
+      info->parent.caps = caps;
     }
 
-    if (gst_structure_get_int (caps_st, "width", &tmp) &&
-        gst_structure_get_int (caps_st, "height", &tmp2)) {
+    if (gst_video_format_parse_caps (caps, &format, &tmp, &tmp2)) {
       info->width = (guint) tmp;
       info->height = (guint) tmp2;
     }
@@ -716,73 +608,34 @@ collect_information (GstDiscoverer * dc, const GstStructure * st,
     if (gst_structure_get_int (caps_st, "depth", &tmp))
       info->depth = (guint) tmp;
 
-    if (gst_structure_get_fraction (caps_st, "pixel-aspect-ratio", &tmp, &tmp2)) {
+    if (gst_video_parse_caps_pixel_aspect_ratio (caps, &tmp, &tmp2)) {
       info->par_num = tmp;
       info->par_denom = tmp2;
-    } else {
-      info->par_num = 1;
-      info->par_denom = 1;
     }
 
-    if (gst_structure_get_fraction (caps_st, "framerate", &tmp, &tmp2)) {
+    if (gst_video_parse_caps_framerate (caps, &tmp, &tmp2)) {
       info->framerate_num = tmp;
       info->framerate_denom = tmp2;
     }
 
-    if (!gst_structure_get_boolean (caps_st, "interlaced", &info->interlaced))
-      info->interlaced = FALSE;
+    if (gst_video_format_parse_caps_interlaced (caps, &btmp))
+      info->interlaced = btmp;
 
     if (gst_structure_id_has_field (st, _TAGS_QUARK)) {
-      gst_structure_id_get (st, _TAGS_QUARK, GST_TYPE_TAG_LIST, &tags_st, NULL);
-      if (gst_structure_get_uint (tags_st, GST_TAG_BITRATE, &utmp) ||
-          gst_structure_get_uint (tags_st, GST_TAG_NOMINAL_BITRATE, &utmp))
+      gst_structure_id_get (st, _TAGS_QUARK,
+          GST_TYPE_STRUCTURE, &tags_st, NULL);
+      if (gst_structure_get_uint (tags_st, GST_TAG_BITRATE, &utmp))
         info->bitrate = utmp;
 
       if (gst_structure_get_uint (tags_st, GST_TAG_MAXIMUM_BITRATE, &utmp))
         info->max_bitrate = utmp;
 
       /* FIXME: Is it worth it to remove the tags we've parsed? */
-      gst_discoverer_merge_and_replace_tags (&info->parent.tags,
-          (GstTagList *) tags_st);
+      info->parent.tags = gst_tag_list_merge (info->parent.tags,
+          (GstTagList *) tags_st, GST_TAG_MERGE_REPLACE);
+      gst_structure_free (tags_st);
     }
 
-    gst_caps_unref (caps);
-    return (GstDiscovererStreamInfo *) info;
-
-  } else if (is_subtitle_caps (caps)) {
-    GstDiscovererSubtitleInfo *info;
-
-    if (parent)
-      info =
-          (GstDiscovererSubtitleInfo *) gst_discoverer_stream_info_ref (parent);
-    else {
-      info = (GstDiscovererSubtitleInfo *)
-          gst_mini_object_new (GST_TYPE_DISCOVERER_SUBTITLE_INFO);
-      info->parent.caps = gst_caps_ref (caps);
-    }
-
-    if (gst_structure_id_has_field (st, _TAGS_QUARK)) {
-      const gchar *language;
-
-      gst_structure_id_get (st, _TAGS_QUARK, GST_TYPE_TAG_LIST, &tags_st, NULL);
-
-      language = gst_structure_get_string (caps_st, GST_TAG_LANGUAGE_CODE);
-      if (language)
-        info->language = g_strdup (language);
-
-      /* FIXME: Is it worth it to remove the tags we've parsed? */
-      gst_discoverer_merge_and_replace_tags (&info->parent.tags, tags_st);
-    }
-
-    if (!info->language && ((GstDiscovererStreamInfo *) info)->tags) {
-      gchar *language;
-      if (gst_tag_list_get_string (((GstDiscovererStreamInfo *) info)->tags,
-              GST_TAG_LANGUAGE_CODE, &language)) {
-        info->language = language;
-      }
-    }
-
-    gst_caps_unref (caps);
     return (GstDiscovererStreamInfo *) info;
 
   } else {
@@ -790,19 +643,20 @@ collect_information (GstDiscoverer * dc, const GstStructure * st,
     GstDiscovererStreamInfo *info;
 
     if (parent)
-      info = gst_discoverer_stream_info_ref (parent);
+      info = parent;
     else {
       info = (GstDiscovererStreamInfo *)
           gst_mini_object_new (GST_TYPE_DISCOVERER_STREAM_INFO);
-      info->caps = gst_caps_ref (caps);
+      info->caps = caps;
     }
 
     if (gst_structure_id_get (st, _TAGS_QUARK,
-            GST_TYPE_TAG_LIST, &tags_st, NULL)) {
-      gst_discoverer_merge_and_replace_tags (&info->tags, tags_st);
+            GST_TYPE_STRUCTURE, &tags_st, NULL)) {
+      info->tags = gst_tag_list_merge (info->tags, (GstTagList *) tags_st,
+          GST_TAG_MERGE_REPLACE);
+      gst_structure_free (tags_st);
     }
 
-    gst_caps_unref (caps);
     return info;
   }
 
@@ -826,10 +680,8 @@ find_stream_for_node (GstDiscoverer * dc, const GstStructure * topology)
   gst_structure_id_get (topology, _TOPOLOGY_PAD_QUARK,
       GST_TYPE_PAD, &pad, NULL);
 
-  if (!dc->priv->streams) {
-    gst_object_unref (pad);
+  if (!dc->priv->streams)
     return NULL;
-  }
 
   for (i = 0, tmp = dc->priv->streams; tmp; tmp = tmp->next, i++) {
     ps = (PrivateStream *) tmp->data;
@@ -850,9 +702,9 @@ find_stream_for_node (GstDiscoverer * dc, const GstStructure * topology)
 }
 
 static gboolean
-child_is_raw_stream (const GstCaps * parent, const GstCaps * child)
+child_is_raw_stream (GstCaps * parent, GstCaps * child)
 {
-  const GstStructure *st1, *st2;
+  GstStructure *st1, *st2;
   const gchar *name1, *name2;
 
   st1 = gst_caps_get_structure (parent, 0);
@@ -869,9 +721,6 @@ child_is_raw_stream (const GstCaps * parent, const GstCaps * child)
     return TRUE;
   }
 
-  if (is_subtitle_caps (parent))
-    return TRUE;
-
   return FALSE;
 }
 
@@ -921,29 +770,30 @@ parse_stream_topology (GstDiscoverer * dc, const GstStructure * topology,
           /* We sometimes get an extra sub-stream from the parser. If this is
            * the case, we just replace the parent caps with this stream's caps
            * since they might contain more information */
-          gst_caps_replace (&parent->caps, caps);
+          gst_caps_unref (parent->caps);
+          parent->caps = caps;
 
           parse_stream_topology (dc, st, parent);
           add_to_list = FALSE;
+
         } else if (child_is_raw_stream (parent->caps, caps)) {
           /* This is the "raw" stream corresponding to the parent. This
            * contains more information than the parent, tags etc. */
           parse_stream_topology (dc, st, parent);
           add_to_list = FALSE;
+          gst_caps_unref (caps);
+
         } else {
           GstDiscovererStreamInfo *next = parse_stream_topology (dc, st, NULL);
           res->next = next;
           next->previous = res;
         }
-        gst_caps_unref (caps);
       }
     }
 
     if (add_to_list) {
       dc->priv->current_info->stream_list =
           g_list_append (dc->priv->current_info->stream_list, res);
-    } else {
-      gst_discoverer_stream_info_unref (res);
     }
 
   } else if (GST_VALUE_HOLDS_LIST (nval)) {
@@ -964,21 +814,12 @@ parse_stream_topology (GstDiscoverer * dc, const GstStructure * topology,
     res = (GstDiscovererStreamInfo *) cont;
 
     if (gst_structure_id_has_field (topology, _TAGS_QUARK)) {
-      GstTagList *tmp;
-
       gst_structure_id_get (topology, _TAGS_QUARK,
-          GST_TYPE_TAG_LIST, &tags, NULL);
-
-      GST_DEBUG ("Merge tags %" GST_PTR_FORMAT, tags);
-
-      tmp =
+          GST_TYPE_STRUCTURE, &tags, NULL);
+      cont->parent.tags =
           gst_tag_list_merge (cont->parent.tags, (GstTagList *) tags,
           GST_TAG_MERGE_APPEND);
       gst_tag_list_free (tags);
-      if (cont->parent.tags)
-        gst_tag_list_free (cont->parent.tags);
-      cont->parent.tags = tmp;
-      GST_DEBUG ("Container info tags %" GST_PTR_FORMAT, tmp);
     }
 
     for (i = 0; i < len; i++) {
@@ -1015,55 +856,17 @@ discoverer_collect (GstDiscoverer * dc)
   if (dc->priv->streams) {
     /* FIXME : Make this querying optional */
     if (TRUE) {
-      GstElement *pipeline = (GstElement *) dc->priv->pipeline;
       GstFormat format = GST_FORMAT_TIME;
       gint64 dur;
 
       GST_DEBUG ("Attempting to query duration");
 
-      if (gst_element_query_duration (pipeline, &format, &dur)) {
+      if (gst_element_query_duration ((GstElement *) dc->priv->pipeline,
+              &format, &dur)) {
         if (format == GST_FORMAT_TIME) {
           GST_DEBUG ("Got duration %" GST_TIME_FORMAT, GST_TIME_ARGS (dur));
           dc->priv->current_info->duration = (guint64) dur;
         }
-      } else {
-        GstStateChangeReturn sret;
-
-        /* Some parsers may not even return a rough estimate right away, e.g.
-         * because they've only processed a single frame so far, so if we
-         * didn't get a duration the first time, spin a bit and try again.
-         * Ugly, but still better than making parsers or other elements return
-         * completely bogus values. We need some API extensions to solve this
-         * better. */
-        GST_INFO ("No duration yet, try a bit harder..");
-        sret = gst_element_set_state (pipeline, GST_STATE_PLAYING);
-        if (sret != GST_STATE_CHANGE_FAILURE) {
-          int i;
-
-          for (i = 0; i < 2; ++i) {
-            g_usleep (G_USEC_PER_SEC / 20);
-            if (gst_element_query_duration (pipeline, &format, &dur) &&
-                format == GST_FORMAT_TIME && dur > 0) {
-              GST_DEBUG ("Got duration %" GST_TIME_FORMAT, GST_TIME_ARGS (dur));
-              dc->priv->current_info->duration = (guint64) dur;
-              break;
-            }
-          }
-          gst_element_set_state (pipeline, GST_STATE_PAUSED);
-        }
-      }
-
-      if (dc->priv->seeking_query) {
-        if (gst_element_query (pipeline, dc->priv->seeking_query)) {
-          gboolean seekable;
-
-          gst_query_parse_seeking (dc->priv->seeking_query, &format,
-              &seekable, NULL, NULL);
-          if (format == GST_FORMAT_TIME) {
-            GST_DEBUG ("Got seekable %d", seekable);
-            dc->priv->current_info->seekable = seekable;
-          }
-        }
       }
     }
 
@@ -1076,7 +879,7 @@ discoverer_collect (GstDiscoverer * dc)
      * caps named image/<foo> (th exception being MJPEG video which is also
      * type image/jpeg), and should consist of precisely one stream (actually
      * initially there are 2, the image and raw stream, but we squash these
-     * while parsing the stream topology). At some point, if we find that these
+     * while parsing the stream topology). At some ponit, if we find that these
      * conditions are not sufficient, we can count the number of decoders and
      * parsers in the chain, and if there's more than one decoder, or any
      * parser at all, we should not mark this as an image.
@@ -1088,8 +891,8 @@ discoverer_collect (GstDiscoverer * dc)
           gst_caps_get_structure (dc->priv->current_info->stream_info->caps, 0);
 
       if (g_str_has_prefix (gst_structure_get_name (st), "image/"))
-        ((GstDiscovererVideoInfo *) dc->priv->current_info->stream_info)->
-            is_image = TRUE;
+        ((GstDiscovererVideoInfo *) dc->priv->current_info->
+            stream_info)->is_image = TRUE;
     }
   }
 
@@ -1102,34 +905,14 @@ discoverer_collect (GstDiscoverer * dc)
   }
 }
 
-static void
-get_async_cb (gpointer cb_data, GSource * source, GSourceFunc * func,
-    gpointer * data)
-{
-  *func = (GSourceFunc) async_timeout_cb;
-  *data = cb_data;
-}
-
-/* Wrapper since GSourceCallbackFuncs don't expect a return value from ref() */
-static void
-_void_g_object_ref (gpointer object)
-{
-  g_object_ref (G_OBJECT (object));
-}
-
 static void
 handle_current_async (GstDiscoverer * dc)
 {
   GSource *source;
-  static GSourceCallbackFuncs cb_funcs = {
-    _void_g_object_ref,
-    g_object_unref,
-    get_async_cb,
-  };
 
   /* Attach a timeout to the main context */
   source = g_timeout_source_new (dc->priv->timeout / GST_MSECOND);
-  g_source_set_callback_indirect (source, g_object_ref (dc), &cb_funcs);
+  g_source_set_callback (source, (GSourceFunc) async_timeout_cb, dc, NULL);
   dc->priv->timeoutid = g_source_attach (source, dc->priv->ctx);
   g_source_unref (source);
 }
@@ -1141,8 +924,7 @@ handle_message (GstDiscoverer * dc, GstMessage * msg)
 {
   gboolean done = FALSE;
 
-  GST_DEBUG_OBJECT (GST_MESSAGE_SRC (msg), "got a %s message",
-      GST_MESSAGE_TYPE_NAME (msg));
+  GST_DEBUG ("got a %s message", GST_MESSAGE_TYPE_NAME (msg));
 
   switch (GST_MESSAGE_TYPE (msg)) {
     case GST_MESSAGE_ERROR:{
@@ -1150,8 +932,8 @@ handle_message (GstDiscoverer * dc, GstMessage * msg)
       gchar *debug;
 
       gst_message_parse_error (msg, &gerr, &debug);
-      GST_WARNING_OBJECT (GST_MESSAGE_SRC (msg),
-          "Got an error [debug:%s], [message:%s]", debug, gerr->message);
+      GST_WARNING ("Got an error [debug:%s]", debug);
+      GST_WARNING ("Got an error [message:%s]", gerr->message);
       dc->priv->current_error = gerr;
       g_free (debug);
 
@@ -1182,15 +964,10 @@ handle_message (GstDiscoverer * dc, GstMessage * msg)
       GST_DEBUG_OBJECT (GST_MESSAGE_SRC (msg),
           "structure %" GST_PTR_FORMAT, msg->structure);
       if (sttype == _MISSING_PLUGIN_QUARK) {
-        GST_DEBUG_OBJECT (GST_MESSAGE_SRC (msg),
-            "Setting result to MISSING_PLUGINS");
+        GST_DEBUG ("Setting result to MISSING_PLUGINS");
         dc->priv->current_info->result = GST_DISCOVERER_MISSING_PLUGINS;
-        if (dc->priv->current_info->misc)
-          gst_structure_free (dc->priv->current_info->misc);
         dc->priv->current_info->misc = gst_structure_copy (msg->structure);
       } else if (sttype == _STREAM_TOPOLOGY_QUARK) {
-        if (dc->priv->current_topology)
-          gst_structure_free (dc->priv->current_topology);
         dc->priv->current_topology = gst_structure_copy (msg->structure);
       }
     }
@@ -1198,20 +975,15 @@ handle_message (GstDiscoverer * dc, GstMessage * msg)
 
     case GST_MESSAGE_TAG:
     {
-      GstTagList *tl, *tmp;
+      GstTagList *tl;
 
       gst_message_parse_tag (msg, &tl);
-      GST_DEBUG_OBJECT (GST_MESSAGE_SRC (msg), "Got tags %" GST_PTR_FORMAT, tl);
+      GST_DEBUG ("Got tags %" GST_PTR_FORMAT, tl);
       /* Merge with current tags */
-      tmp =
+      dc->priv->current_info->tags =
           gst_tag_list_merge (dc->priv->current_info->tags, tl,
           GST_TAG_MERGE_APPEND);
       gst_tag_list_free (tl);
-      if (dc->priv->current_info->tags)
-        gst_tag_list_free (dc->priv->current_info->tags);
-      dc->priv->current_info->tags = tmp;
-      GST_DEBUG_OBJECT (GST_MESSAGE_SRC (msg), "Current info %p, tags %"
-          GST_PTR_FORMAT, dc->priv->current_info, tmp);
     }
       break;
 
@@ -1241,6 +1013,7 @@ handle_current_sync (GstDiscoverer * dc)
       done = handle_message (dc, msg);
       gst_message_unref (msg);
     }
+
   } while (!done && (g_timer_elapsed (timer, NULL) < deadline));
 
   /* return result */
@@ -1333,10 +1106,7 @@ discoverer_bus_cb (GstBus * bus, GstMessage * msg, GstDiscoverer * dc)
   if (dc->priv->processing) {
     if (handle_message (dc, msg)) {
       GST_DEBUG ("Stopping asynchronously");
-      /* Serialise with _event_probe() */
-      DISCO_LOCK (dc);
       dc->priv->processing = FALSE;
-      DISCO_UNLOCK (dc);
       discoverer_collect (dc);
       discoverer_cleanup (dc);
     }
@@ -1346,14 +1116,12 @@ discoverer_bus_cb (GstBus * bus, GstMessage * msg, GstDiscoverer * dc)
 static gboolean
 async_timeout_cb (GstDiscoverer * dc)
 {
-  if (!g_source_is_destroyed (g_main_current_source ())) {
-    dc->priv->timeoutid = 0;
-    GST_DEBUG ("Setting result to TIMEOUT");
-    dc->priv->current_info->result = GST_DISCOVERER_TIMEOUT;
-    dc->priv->processing = FALSE;
-    discoverer_collect (dc);
-    discoverer_cleanup (dc);
-  }
+  dc->priv->timeoutid = 0;
+  GST_DEBUG ("Setting result to TIMEOUT");
+  dc->priv->current_info->result = GST_DISCOVERER_TIMEOUT;
+  dc->priv->processing = FALSE;
+  discoverer_collect (dc);
+  discoverer_cleanup (dc);
   return FALSE;
 }
 
@@ -1404,11 +1172,11 @@ beach:
 /**
  * gst_discoverer_start:
  * @discoverer: A #GstDiscoverer
- *
+ * 
  * Allow asynchronous discovering of URIs to take place.
  * A #GMainLoop must be available for #GstDiscoverer to properly work in
  * asynchronous mode.
- *
+ * 
  * Since: 0.10.31
  */
 void
@@ -1427,8 +1195,9 @@ gst_discoverer_start (GstDiscoverer * discoverer)
   discoverer->priv->async = TRUE;
   discoverer->priv->running = TRUE;
 
+#if GLIB_CHECK_VERSION(2, 22, 0)
   ctx = g_main_context_get_thread_default ();
-
+#endif
   /* Connect to bus signals */
   if (ctx == NULL)
     ctx = g_main_context_default ();
@@ -1450,8 +1219,8 @@ gst_discoverer_start (GstDiscoverer * discoverer)
  *
  * Stop the discovery of any pending URIs and clears the list of
  * pending URIS (if any).
- *
- * Since: 0.10.31
+ * 
+ * Since 0.10.31
  */
 void
 gst_discoverer_stop (GstDiscoverer * discoverer)
@@ -1469,11 +1238,9 @@ gst_discoverer_stop (GstDiscoverer * discoverer)
     /* We prevent any further processing by setting the bus to
      * flushing and setting the pipeline to READY.
      * _reset() will take care of the rest of the cleanup */
-    if (discoverer->priv->bus)
-      gst_bus_set_flushing (discoverer->priv->bus, TRUE);
-    if (discoverer->priv->pipeline)
-      gst_element_set_state ((GstElement *) discoverer->priv->pipeline,
-          GST_STATE_READY);
+    gst_bus_set_flushing (discoverer->priv->bus, TRUE);
+    gst_element_set_state ((GstElement *) discoverer->priv->pipeline,
+        GST_STATE_READY);
   }
   discoverer->priv->running = FALSE;
   DISCO_UNLOCK (discoverer);
@@ -1509,12 +1276,11 @@ gst_discoverer_stop (GstDiscoverer * discoverer)
  * discovery of the @uri will only take place if gst_discoverer_start() has
  * been called.
  *
- * A copy of @uri will be made internally, so the caller can safely g_free()
- * afterwards.
+ * A copy of @uri will be done internally, the caller can safely g_free() afterwards.
  *
- * Returns: %TRUE if the @uri was successfully appended to the list of pending
+ * Returns: %TRUE if the @uri was succesfully appended to the list of pending
  * uris, else %FALSE
- *
+ * 
  * Since: 0.10.31
  */
 gboolean
@@ -1543,16 +1309,15 @@ gst_discoverer_discover_uri_async (GstDiscoverer * discoverer,
  * gst_discoverer_discover_uri:
  * @discoverer: A #GstDiscoverer
  * @uri: The URI to run on.
- * @err: (out) (allow-none): If an error occurred, this field will be filled in.
+ * @err: If an error occured, this field will be filled in.
  *
  * Synchronously discovers the given @uri.
  *
- * A copy of @uri will be made internally, so the caller can safely g_free()
- * afterwards.
- *
- * Returns: (transfer full): the result of the scanning. Can be %NULL if an
- * error occurred.
+ * A copy of @uri will be done internally, the caller can safely g_free() afterwards.
  *
+ * Returns: (transfer none): see #GstDiscovererInfo. The caller must free this structure
+ * after use.
+ * 
  * Since: 0.10.31
  */
 GstDiscovererInfo *
@@ -1579,12 +1344,10 @@ gst_discoverer_discover_uri (GstDiscoverer * discoverer, const gchar * uri,
   discoverer_collect (discoverer);
 
   /* Get results */
-  if (err) {
-    if (discoverer->priv->current_error)
-      *err = g_error_copy (discoverer->priv->current_error);
-    else
-      *err = NULL;
-  }
+  if (discoverer->priv->current_error)
+    *err = g_error_copy (discoverer->priv->current_error);
+  else
+    *err = NULL;
   if (res != GST_DISCOVERER_OK) {
     GST_DEBUG ("Setting result to %d (was %d)", res,
         discoverer->priv->current_info->result);
@@ -1605,11 +1368,11 @@ gst_discoverer_discover_uri (GstDiscoverer * discoverer, const gchar * uri,
  *
  * Creates a new #GstDiscoverer with the provided timeout.
  *
- * Returns: (transfer full): The new #GstDiscoverer.
- * If an error occurred when creating the discoverer, @err will be set
+ * Returns: The new #GstDiscoverer. Free with gst_object_unref() when done.
+ * If an error happened when creating the discoverer, @err will be set
  * accordingly and %NULL will be returned. If @err is set, the caller must
  * free it when no longer needed using g_error_free().
- *
+ * 
  * Since: 0.10.31
  */
 GstDiscoverer *
