@@ -22,9 +22,13 @@
 
 #include <gst/gst.h>
 
+#include <gst/pbutils/gstpluginsbaseversion.h>
 #include <gst/pbutils/descriptions.h>
 #include <gst/pbutils/missing-plugins.h>
 #include <gst/pbutils/install-plugins.h>
+#include <gst/pbutils/codec-utils.h>
+#include <gst/pbutils/pbutils-enumtypes.h>
+#include <gst/pbutils/gstdiscoverer.h>
 
 G_BEGIN_DECLS
 
