@@ -46,6 +46,10 @@ struct _GstBaseRTPPayloadPrivate
   gboolean ssrc_random;
   guint16 next_seqnum;
   gboolean perfect_rtptime;
+  gint notified_first_timestamp;
+
+  gint64 prop_max_ptime;
+  gint64 caps_max_ptime;
 };
 
 /* BaseRTPPayload signals and args */
@@ -69,6 +73,7 @@ enum
 #define DEFAULT_MAX_PTIME               -1
 #define DEFAULT_MIN_PTIME               0
 #define DEFAULT_PERFECT_RTPTIME         TRUE
+#define DEFAULT_PTIME_MULTIPLE          0
 
 enum
 {
@@ -83,6 +88,7 @@ enum
   PROP_TIMESTAMP,
   PROP_SEQNUM,
   PROP_PERFECT_RTPTIME,
+  PROP_PTIME_MULTIPLE,
   PROP_LAST
 };
 
@@ -222,6 +228,18 @@ gst_basertppayload_class_init (GstBaseRTPPayloadClass * klass)
       g_param_spec_boolean ("perfect-rtptime", "Perfect RTP Time",
           "Generate perfect RTP timestamps when possible",
           DEFAULT_PERFECT_RTPTIME, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  /**
+   * GstBaseRTPAudioPayload:ptime-multiple:
+   *
+   * Force buffers to be multiples of this duration in ns (0 disables)
+   *
+   * Since: 0.10.29
+   **/
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_PTIME_MULTIPLE,
+      g_param_spec_int64 ("ptime-multiple", "Packet time multiple",
+          "Force buffers to be multiples of this duration in ns (0 disables)",
+          0, G_MAXINT64, DEFAULT_PTIME_MULTIPLE,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gstelement_class->change_state = gst_basertppayload_change_state;
 
@@ -276,11 +294,15 @@ gst_basertppayload_init (GstBaseRTPPayload * basertppayload, gpointer g_class)
   basertppayload->max_ptime = DEFAULT_MAX_PTIME;
   basertppayload->min_ptime = DEFAULT_MIN_PTIME;
   basertppayload->priv->perfect_rtptime = DEFAULT_PERFECT_RTPTIME;
+  basertppayload->abidata.ABI.ptime_multiple = DEFAULT_PTIME_MULTIPLE;
 
   basertppayload->media = NULL;
   basertppayload->encoding_name = NULL;
 
   basertppayload->clock_rate = 0;
+
+  basertppayload->priv->caps_max_ptime = DEFAULT_MAX_PTIME;
+  basertppayload->priv->prop_max_ptime = DEFAULT_MAX_PTIME;
 }
 
 static void
@@ -447,7 +469,8 @@ no_function:
  */
 void
 gst_basertppayload_set_options (GstBaseRTPPayload * payload,
-    gchar * media, gboolean dynamic, gchar * encoding_name, guint32 clock_rate)
+    const gchar * media, gboolean dynamic, const gchar * encoding_name,
+    guint32 clock_rate)
 {
   g_return_if_fail (payload != NULL);
   g_return_if_fail (clock_rate != 0);
@@ -469,6 +492,21 @@ copy_fixed (GQuark field_id, const GValue * value, GstStructure * dest)
   return TRUE;
 }
 
+static void
+update_max_ptime (GstBaseRTPPayload * basertppayload)
+{
+  if (basertppayload->priv->caps_max_ptime != -1 &&
+      basertppayload->priv->prop_max_ptime != -1)
+    basertppayload->max_ptime = MIN (basertppayload->priv->caps_max_ptime,
+        basertppayload->priv->prop_max_ptime);
+  else if (basertppayload->priv->caps_max_ptime != -1)
+    basertppayload->max_ptime = basertppayload->priv->caps_max_ptime;
+  else if (basertppayload->priv->prop_max_ptime != -1)
+    basertppayload->max_ptime = basertppayload->priv->prop_max_ptime;
+  else
+    basertppayload->max_ptime = DEFAULT_MAX_PTIME;
+}
+
 /**
  * gst_basertppayload_set_outcaps:
  * @payload: a #GstBaseRTPPayload
@@ -483,8 +521,8 @@ copy_fixed (GQuark field_id, const GValue * value, GstStructure * dest)
  * Returns: %TRUE if the caps could be set.
  */
 gboolean
-gst_basertppayload_set_outcaps (GstBaseRTPPayload * payload, gchar * fieldname,
-    ...)
+gst_basertppayload_set_outcaps (GstBaseRTPPayload * payload,
+    const gchar * fieldname, ...)
 {
   GstCaps *srccaps, *peercaps;
   gboolean res;
@@ -508,6 +546,9 @@ gst_basertppayload_set_outcaps (GstBaseRTPPayload * payload, gchar * fieldname,
     GST_DEBUG_OBJECT (payload, "custom added: %" GST_PTR_FORMAT, srccaps);
   }
 
+  payload->priv->caps_max_ptime = DEFAULT_MAX_PTIME;
+  payload->abidata.ABI.ptime = 0;
+
   /* the peer caps can override some of the defaults */
   peercaps = gst_pad_peer_get_caps (payload->srcpad);
   if (peercaps == NULL) {
@@ -524,6 +565,7 @@ gst_basertppayload_set_outcaps (GstBaseRTPPayload * payload, gchar * fieldname,
     GstStructure *s, *d;
     const GValue *value;
     gint pt;
+    guint max_ptime, ptime;
 
     /* peer provides caps we can use to fixate, intersect. This always returns a
      * writable caps. */
@@ -531,12 +573,23 @@ gst_basertppayload_set_outcaps (GstBaseRTPPayload * payload, gchar * fieldname,
     gst_caps_unref (srccaps);
     gst_caps_unref (peercaps);
 
+    if (gst_caps_is_empty (temp)) {
+      gst_caps_unref (temp);
+      return FALSE;
+    }
+
     /* now fixate, start by taking the first caps */
     gst_caps_truncate (temp);
 
     /* get first structure */
     s = gst_caps_get_structure (temp, 0);
 
+    if (gst_structure_get_uint (s, "maxptime", &max_ptime))
+      payload->priv->caps_max_ptime = max_ptime * GST_MSECOND;
+
+    if (gst_structure_get_uint (s, "ptime", &ptime))
+      payload->abidata.ABI.ptime = ptime * GST_MSECOND;
+
     if (gst_structure_get_int (s, "payload", &pt)) {
       /* use peer pt */
       GST_BASE_RTP_PAYLOAD_PT (payload) = pt;
@@ -602,6 +655,8 @@ gst_basertppayload_set_outcaps (GstBaseRTPPayload * payload, gchar * fieldname,
     GST_DEBUG_OBJECT (payload, "with peer caps: %" GST_PTR_FORMAT, srccaps);
   }
 
+  update_max_ptime (payload);
+
   res = gst_pad_set_caps (GST_BASE_RTP_PAYLOAD_SRCPAD (payload), srccaps);
   gst_caps_unref (srccaps);
 
@@ -651,8 +706,9 @@ find_timestamp (GstBuffer ** buffer, guint group, guint idx, HeaderData * data)
   data->timestamp = GST_BUFFER_TIMESTAMP (*buffer);
   data->offset = GST_BUFFER_OFFSET (*buffer);
 
-  /* stop when we find a timestamp and duration */
-  if (data->timestamp != -1 && data->offset != -1)
+  /* stop when we find a timestamp. We take whatever offset is associated with
+   * the timestamp (if any) to do perfect timestamps when we need to. */
+  if (data->timestamp != -1)
     return GST_BUFFER_LIST_END;
   else
     return GST_BUFFER_LIST_CONTINUE;
@@ -696,10 +752,11 @@ gst_basertppayload_prepare_push (GstBaseRTPPayload * payload,
   data.ssrc = payload->current_ssrc;
   data.pt = payload->pt;
   data.caps = GST_PAD_CAPS (payload->srcpad);
-  data.timestamp = -1;
 
   /* find the first buffer with a timestamp */
   if (is_list) {
+    data.timestamp = -1;
+    data.offset = GST_BUFFER_OFFSET_NONE;
     gst_buffer_list_foreach (GST_BUFFER_LIST_CAST (obj),
         (GstBufferListFunc) find_timestamp, &data);
   } else {
@@ -753,6 +810,12 @@ gst_basertppayload_prepare_push (GstBaseRTPPayload * payload,
       GST_BUFFER_SIZE (GST_BUFFER (obj)), payload->seqnum, data.rtptime,
       GST_TIME_ARGS (data.timestamp));
 
+  if (g_atomic_int_compare_and_exchange (&payload->priv->
+          notified_first_timestamp, 1, 0)) {
+    g_object_notify (G_OBJECT (payload), "timestamp");
+    g_object_notify (G_OBJECT (payload), "seqnum");
+  }
+
   return GST_FLOW_OK;
 
   /* ERRORS */
@@ -856,7 +919,8 @@ gst_basertppayload_set_property (GObject * object, guint prop_id,
           basertppayload->seqnum_offset, priv->seqnum_offset_random);
       break;
     case PROP_MAX_PTIME:
-      basertppayload->max_ptime = g_value_get_int64 (value);
+      basertppayload->priv->prop_max_ptime = g_value_get_int64 (value);
+      update_max_ptime (basertppayload);
       break;
     case PROP_MIN_PTIME:
       basertppayload->min_ptime = g_value_get_int64 (value);
@@ -864,6 +928,9 @@ gst_basertppayload_set_property (GObject * object, guint prop_id,
     case PROP_PERFECT_RTPTIME:
       priv->perfect_rtptime = g_value_get_boolean (value);
       break;
+    case PROP_PTIME_MULTIPLE:
+      basertppayload->abidata.ABI.ptime_multiple = g_value_get_int64 (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -920,6 +987,9 @@ gst_basertppayload_get_property (GObject * object, guint prop_id,
     case PROP_PERFECT_RTPTIME:
       g_value_set_boolean (value, priv->perfect_rtptime);
       break;
+    case PROP_PTIME_MULTIPLE:
+      g_value_set_int64 (value, basertppayload->abidata.ABI.ptime_multiple);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -960,6 +1030,7 @@ gst_basertppayload_change_state (GstElement * element,
       else
         basertppayload->ts_base = basertppayload->ts_offset;
       basertppayload->timestamp = basertppayload->ts_base;
+      g_atomic_int_set (&basertppayload->priv->notified_first_timestamp, 1);
       break;
     default:
       break;
@@ -968,6 +1039,9 @@ gst_basertppayload_change_state (GstElement * element,
   ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
 
   switch (transition) {
+    case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
+      g_atomic_int_set (&basertppayload->priv->notified_first_timestamp, 1);
+      break;
     case GST_STATE_CHANGE_READY_TO_NULL:
       break;
     default:
