@@ -231,7 +231,8 @@ gst_rtsp_transport_get_mime (GstRTSPTransMode trans, const gchar ** mime)
  * @manager: location to hold the result
  * @option: option index.
  *
- * Get the #GstElement that can handle the buffers transported over @trans.
+ * Get the #GStreamer element that can handle the buffers transported over
+ * @trans.
  *
  * It is possible that there are several managers available, use @option to
  * selected one.
@@ -268,20 +269,6 @@ parse_mode (GstRTSPTransport * transport, const gchar * str)
   transport->mode_record = (strstr (str, "record") != NULL);
 }
 
-static gboolean
-check_range (const gchar * str, gchar ** tmp, gint * range)
-{
-  glong range_val;
-
-  range_val = strtol (str, tmp, 10);
-  if (range_val >= G_MININT && range_val <= G_MAXINT) {
-    *range = range_val;
-    return TRUE;
-  } else {
-    return FALSE;
-  }
-}
-
 static gboolean
 parse_range (const gchar * str, GstRTSPRange * range)
 {
@@ -299,14 +286,16 @@ parse_range (const gchar * str, GstRTSPRange * range)
     if (g_ascii_isspace (minus[1]) || minus[1] == '+' || minus[1] == '-')
       goto invalid_range;
 
-    if (!check_range (str, &tmp, &range->min) || str == tmp || tmp != minus)
+    range->min = strtol (str, &tmp, 10);
+    if (str == tmp || tmp != minus)
       goto invalid_range;
 
-    if (!check_range (minus + 1, &tmp, &range->max) || (*tmp && *tmp != ';'))
+    range->max = strtol (minus + 1, &tmp, 10);
+    if (*tmp && *tmp != ';')
       goto invalid_range;
   } else {
-    if (!check_range (str, &tmp, &range->min) || str == tmp ||
-        (*tmp && *tmp != ';'))
+    range->min = strtol (str, &tmp, 10);
+    if (str == tmp || (*tmp && *tmp != ';'))
       goto invalid_range;
 
     range->max = -1;
