@@ -772,6 +772,10 @@ gst_video_format_from_fourcc (guint32 fourcc)
       return GST_VIDEO_FORMAT_Y800;
     case GST_MAKE_FOURCC ('Y', '1', '6', ' '):
       return GST_VIDEO_FORMAT_Y16;
+    case GST_MAKE_FOURCC ('U', 'Y', 'V', 'P'):
+      return GST_VIDEO_FORMAT_UYVP;
+    case GST_MAKE_FOURCC ('A', '4', '2', '0'):
+      return GST_VIDEO_FORMAT_A420;
     default:
       return GST_VIDEO_FORMAT_UNKNOWN;
   }
@@ -827,6 +831,10 @@ gst_video_format_to_fourcc (GstVideoFormat format)
       return GST_MAKE_FOURCC ('Y', '8', '0', '0');
     case GST_VIDEO_FORMAT_Y16:
       return GST_MAKE_FOURCC ('Y', '1', '6', ' ');
+    case GST_VIDEO_FORMAT_UYVP:
+      return GST_MAKE_FOURCC ('U', 'Y', 'V', 'P');
+    case GST_VIDEO_FORMAT_A420:
+      return GST_MAKE_FOURCC ('A', '4', '2', '0');
     default:
       return 0;
   }
@@ -960,6 +968,8 @@ gst_video_format_is_rgb (GstVideoFormat format)
     case GST_VIDEO_FORMAT_NV12:
     case GST_VIDEO_FORMAT_NV21:
     case GST_VIDEO_FORMAT_v308:
+    case GST_VIDEO_FORMAT_UYVP:
+    case GST_VIDEO_FORMAT_A420:
       return FALSE;
     case GST_VIDEO_FORMAT_RGBx:
     case GST_VIDEO_FORMAT_BGRx:
@@ -1011,6 +1021,8 @@ gst_video_format_is_yuv (GstVideoFormat format)
     case GST_VIDEO_FORMAT_v308:
     case GST_VIDEO_FORMAT_Y800:
     case GST_VIDEO_FORMAT_Y16:
+    case GST_VIDEO_FORMAT_UYVP:
+    case GST_VIDEO_FORMAT_A420:
       return TRUE;
     case GST_VIDEO_FORMAT_RGBx:
     case GST_VIDEO_FORMAT_BGRx:
@@ -1087,12 +1099,14 @@ gst_video_format_has_alpha (GstVideoFormat format)
     case GST_VIDEO_FORMAT_v308:
     case GST_VIDEO_FORMAT_Y800:
     case GST_VIDEO_FORMAT_Y16:
+    case GST_VIDEO_FORMAT_UYVP:
       return FALSE;
     case GST_VIDEO_FORMAT_AYUV:
     case GST_VIDEO_FORMAT_RGBA:
     case GST_VIDEO_FORMAT_BGRA:
     case GST_VIDEO_FORMAT_ARGB:
     case GST_VIDEO_FORMAT_ABGR:
+    case GST_VIDEO_FORMAT_A420:
       return TRUE;
     case GST_VIDEO_FORMAT_RGBx:
     case GST_VIDEO_FORMAT_BGRx:
@@ -1195,6 +1209,14 @@ gst_video_format_get_row_stride (GstVideoFormat format, int component,
     case GST_VIDEO_FORMAT_GRAY16_LE:
     case GST_VIDEO_FORMAT_Y16:
       return GST_ROUND_UP_4 (width * 2);
+    case GST_VIDEO_FORMAT_UYVP:
+      return GST_ROUND_UP_4 ((width * 2 * 5 + 3) / 4);
+    case GST_VIDEO_FORMAT_A420:
+      if (component == 0 || component == 3) {
+        return GST_ROUND_UP_4 (width);
+      } else {
+        return GST_ROUND_UP_4 (GST_ROUND_UP_2 (width) / 2);
+      }
     default:
       return 0;
   }
@@ -1226,6 +1248,7 @@ gst_video_format_get_pixel_stride (GstVideoFormat format, int component)
     case GST_VIDEO_FORMAT_Y41B:
     case GST_VIDEO_FORMAT_Y42B:
     case GST_VIDEO_FORMAT_Y444:
+    case GST_VIDEO_FORMAT_A420:
       return 1;
     case GST_VIDEO_FORMAT_YUY2:
     case GST_VIDEO_FORMAT_YVYU:
@@ -1277,6 +1300,9 @@ gst_video_format_get_pixel_stride (GstVideoFormat format, int component)
     case GST_VIDEO_FORMAT_GRAY16_LE:
     case GST_VIDEO_FORMAT_Y16:
       return 2;
+    case GST_VIDEO_FORMAT_UYVP:
+      /* UYVP is packed at the bit level, so pixel stride doesn't make sense */
+      return 0;
     default:
       return 0;
   }
@@ -1313,6 +1339,9 @@ gst_video_format_get_component_width (GstVideoFormat format,
     case GST_VIDEO_FORMAT_Y42B:
     case GST_VIDEO_FORMAT_v210:
     case GST_VIDEO_FORMAT_v216:
+    case GST_VIDEO_FORMAT_NV12:
+    case GST_VIDEO_FORMAT_NV21:
+    case GST_VIDEO_FORMAT_UYVP:
       if (component == 0) {
         return width;
       } else {
@@ -1341,14 +1370,18 @@ gst_video_format_get_component_width (GstVideoFormat format,
     case GST_VIDEO_FORMAT_BGR15:
     case GST_VIDEO_FORMAT_Y444:
     case GST_VIDEO_FORMAT_v308:
-    case GST_VIDEO_FORMAT_NV12:
-    case GST_VIDEO_FORMAT_NV21:
     case GST_VIDEO_FORMAT_GRAY8:
     case GST_VIDEO_FORMAT_GRAY16_BE:
     case GST_VIDEO_FORMAT_GRAY16_LE:
     case GST_VIDEO_FORMAT_Y800:
     case GST_VIDEO_FORMAT_Y16:
       return width;
+    case GST_VIDEO_FORMAT_A420:
+      if (component == 0 || component == 1) {
+        return width;
+      } else {
+        return GST_ROUND_UP_2 (width) / 2;
+      }
     default:
       return 0;
   }
@@ -1415,7 +1448,14 @@ gst_video_format_get_component_height (GstVideoFormat format,
     case GST_VIDEO_FORMAT_GRAY16_LE:
     case GST_VIDEO_FORMAT_Y800:
     case GST_VIDEO_FORMAT_Y16:
+    case GST_VIDEO_FORMAT_UYVP:
       return height;
+    case GST_VIDEO_FORMAT_A420:
+      if (component == 0 || component == 1) {
+        return height;
+      } else {
+        return GST_ROUND_UP_2 (height) / 2;
+      }
     default:
       return 0;
   }
@@ -1615,6 +1655,25 @@ gst_video_format_get_component_offset (GstVideoFormat format,
     case GST_VIDEO_FORMAT_Y800:
     case GST_VIDEO_FORMAT_Y16:
       return 0;
+    case GST_VIDEO_FORMAT_UYVP:
+      /* UYVP is bit-packed, so this doesn't make sense */
+      return 0;
+    case GST_VIDEO_FORMAT_A420:
+      if (component == 0)
+        return 0;
+      if (component == 1)
+        return GST_ROUND_UP_4 (width) * GST_ROUND_UP_2 (height);
+      if (component == 2) {
+        return GST_ROUND_UP_4 (width) * GST_ROUND_UP_2 (height) +
+            GST_ROUND_UP_4 (GST_ROUND_UP_2 (width) / 2) *
+            (GST_ROUND_UP_2 (height) / 2);
+      }
+      if (component == 3) {
+        return GST_ROUND_UP_4 (width) * GST_ROUND_UP_2 (height) +
+            2 * GST_ROUND_UP_4 (GST_ROUND_UP_2 (width) / 2) *
+            (GST_ROUND_UP_2 (height) / 2);
+
+      }
     default:
       return 0;
   }
@@ -1693,6 +1752,13 @@ gst_video_format_get_size (GstVideoFormat format, int width, int height)
     case GST_VIDEO_FORMAT_GRAY16_LE:
     case GST_VIDEO_FORMAT_Y16:
       return GST_ROUND_UP_4 (width * 2) * height;
+    case GST_VIDEO_FORMAT_UYVP:
+      return GST_ROUND_UP_4 ((width * 2 * 5 + 3) / 4) * height;
+    case GST_VIDEO_FORMAT_A420:
+      size = 2 * GST_ROUND_UP_4 (width) * GST_ROUND_UP_2 (height);
+      size += GST_ROUND_UP_4 (GST_ROUND_UP_2 (width) / 2) *
+          (GST_ROUND_UP_2 (height) / 2) * 2;
+      return size;
     default:
       return 0;
   }
