@@ -23,6 +23,11 @@
  * Playbin provides a stand-alone everything-in-one abstraction for an
  * audio and/or video player.
  *
+ * <note>
+ * This element is deprecated and no longer supported. You should use
+ * the #playbin2 element instead.
+ * </note>
+ *
  * It can handle both audio and video files and features
  * <itemizedlist>
  * <listitem>
@@ -217,6 +222,8 @@
  * ]| This will play back title 1 of a DVD in your disc drive (assuming
  * the drive is detected automatically by the plugin).
  * </refsect2>
+ *
+ * Deprecated: use playbin2 instead
  */
 
 #ifdef HAVE_CONFIG_H
@@ -230,6 +237,7 @@
 #include <gst/pbutils/pbutils.h>
 
 #include "gstplaybasebin.h"
+#include "gstplayback.h"
 
 GST_DEBUG_CATEGORY_STATIC (gst_play_bin_debug);
 #define GST_CAT_DEFAULT gst_play_bin_debug
@@ -332,12 +340,6 @@ static GstElementClass *parent_class;
 
 //static guint gst_play_bin_signals[LAST_SIGNAL] = { 0 };
 
-static const GstElementDetails gst_play_bin_details =
-GST_ELEMENT_DETAILS ("Player Bin",
-    "Generic/Bin/Player",
-    "Autoplug and play media from an uri",
-    "Wim Taymans <wim.taymans@gmail.com>");
-
 static GType
 gst_play_bin_get_type (void)
 {
@@ -416,7 +418,10 @@ gst_play_bin_class_init (GstPlayBinClass * klass)
 
   gobject_klass->dispose = gst_play_bin_dispose;
 
-  gst_element_class_set_details (gstelement_klass, &gst_play_bin_details);
+  gst_element_class_set_details_simple (gstelement_klass,
+      "Player Bin", "Generic/Bin/Player",
+      "Autoplug and play media from an uri",
+      "Wim Taymans <wim.taymans@gmail.com>");
 
   gstelement_klass->change_state =
       GST_DEBUG_FUNCPTR (gst_play_bin_change_state);
@@ -485,6 +490,10 @@ gst_play_bin_dispose (GObject * object)
     gst_object_unref (play_bin->textoverlay_element);
     play_bin->textoverlay_element = NULL;
   }
+  if (play_bin->volume_element) {
+    gst_object_unref (play_bin->volume_element);
+    play_bin->volume_element = NULL;
+  }
   if (play_bin->spu_element != NULL) {
     gst_object_unref (play_bin->spu_element);
     play_bin->spu_element = NULL;
@@ -641,6 +650,10 @@ gst_play_bin_set_property (GObject * object, guint prop_id,
       if (play_bin->audio_sink != NULL) {
         gst_object_unref (play_bin->audio_sink);
       }
+      if (play_bin->volume_element != NULL) {
+        gst_object_unref (play_bin->volume_element);
+        play_bin->volume_element = NULL;
+      }
       play_bin->audio_sink = g_value_get_object (value);
       if (play_bin->audio_sink != NULL) {
         gst_object_ref (play_bin->audio_sink);
@@ -839,7 +852,7 @@ gen_video_element (GstPlayBin * play_bin)
       goto no_sinks;
   }
   gst_object_ref (sink);
-  g_hash_table_insert (play_bin->cache, "video_sink", sink);
+  g_hash_table_insert (play_bin->cache, (gpointer) "video_sink", sink);
 
   /* create a bin to hold objects, as we create them we add them to this bin so
    * that when something goes wrong we only need to unref the bin */
@@ -877,7 +890,7 @@ gen_video_element (GstPlayBin * play_bin)
   /* since we're gonna add it to a bin but don't want to lose it,
    * we keep a reference. */
   gst_object_ref (element);
-  g_hash_table_insert (play_bin->cache, "vbin", element);
+  g_hash_table_insert (play_bin->cache, (gpointer) "vbin", element);
 
   return element;
 
@@ -1108,7 +1121,7 @@ gen_audio_element (GstPlayBin * play_bin)
   }
 
   gst_object_ref (sink);
-  g_hash_table_insert (play_bin->cache, "audio_sink", sink);
+  g_hash_table_insert (play_bin->cache, (gpointer) "audio_sink", sink);
 
   element = gst_bin_new ("abin");
   gst_bin_add (GST_BIN_CAST (element), sink);
@@ -1145,7 +1158,7 @@ gen_audio_element (GstPlayBin * play_bin)
   /* since we're gonna add it to a bin but don't want to lose it,
    * we keep a reference. */
   gst_object_ref (element);
-  g_hash_table_insert (play_bin->cache, "abin", element);
+  g_hash_table_insert (play_bin->cache, (gpointer) "abin", element);
 
   return element;
 
@@ -1442,11 +1455,6 @@ remove_sinks (GstPlayBin * play_bin)
     gst_object_unref (play_bin->textoverlay_element);
     play_bin->textoverlay_element = NULL;
   }
-
-  if (play_bin->volume_element) {
-    gst_object_unref (play_bin->volume_element);
-    play_bin->volume_element = NULL;
-  }
 }
 
 /* loop over the streams and set up the pipeline to play this
