@@ -21,12 +21,7 @@
 #include "config.h"
 #endif
 
-/* FIXME 0.11: suppress warnings for deprecated API such as GStaticRecMutex
- * with newer GLib versions (>= 2.31.0) */
-#define GLIB_DISABLE_DEPRECATION_WARNINGS
-
 #include "gststreamsynchronizer.h"
-#include "gst/glib-compat-private.h"
 
 GST_DEBUG_CATEGORY_STATIC (stream_synchronizer_debug);
 #define GST_CAT_DEFAULT stream_synchronizer_debug
@@ -57,8 +52,6 @@ static GstStaticPadTemplate sinktemplate = GST_STATIC_PAD_TEMPLATE ("sink_%d",
     GST_PAD_REQUEST,
     GST_STATIC_CAPS_ANY);
 
-static const gboolean passthrough = TRUE;
-
 GST_BOILERPLATE (GstStreamSynchronizer, gst_stream_synchronizer,
     GstElement, GST_TYPE_ELEMENT);
 
@@ -94,16 +87,11 @@ gst_stream_get_other_pad (GstStream * stream, GstPad * pad)
 static GstPad *
 gst_stream_get_other_pad_from_pad (GstPad * pad)
 {
-  GstObject *parent = gst_pad_get_parent (pad);
-  GstStreamSynchronizer *self;
+  GstStreamSynchronizer *self =
+      GST_STREAM_SYNCHRONIZER (gst_pad_get_parent (pad));
   GstStream *stream;
   GstPad *opad = NULL;
 
-  /* released pad does not have parent anymore */
-  if (!G_LIKELY (parent))
-    goto exit;
-
-  self = GST_STREAM_SYNCHRONIZER (parent);
   GST_STREAM_SYNCHRONIZER_LOCK (self);
   stream = gst_pad_get_element_private (pad);
   if (!stream)
@@ -115,7 +103,6 @@ out:
   GST_STREAM_SYNCHRONIZER_UNLOCK (self);
   gst_object_unref (self);
 
-exit:
   if (!opad)
     GST_WARNING_OBJECT (pad, "Trying to get other pad after releasing");
 
@@ -203,9 +190,6 @@ gst_stream_synchronizer_src_event (GstPad * pad, GstEvent * event)
   GstPad *opad;
   gboolean ret = FALSE;
 
-  if (passthrough)
-    goto skip_adjustments;
-
   GST_LOG_OBJECT (pad, "Handling event %s: %" GST_PTR_FORMAT,
       GST_EVENT_TYPE_NAME (event), event->structure);
 
@@ -258,8 +242,6 @@ gst_stream_synchronizer_src_event (GstPad * pad, GstEvent * event)
       break;
   }
 
-skip_adjustments:
-
   opad = gst_stream_get_other_pad_from_pad (pad);
   if (opad) {
     ret = gst_pad_push_event (opad, event);
@@ -281,9 +263,6 @@ gst_stream_synchronizer_sink_event (GstPad * pad, GstEvent * event)
   GstPad *opad;
   gboolean ret = FALSE;
 
-  if (passthrough)
-    goto skip_adjustments;
-
   GST_LOG_OBJECT (pad, "Handling event %s: %" GST_PTR_FORMAT,
       GST_EVENT_TYPE_NAME (event), event->structure);
 
@@ -483,15 +462,11 @@ gst_stream_synchronizer_sink_event (GstPad * pad, GstEvent * event)
 
       GST_STREAM_SYNCHRONIZER_LOCK (self);
       stream = gst_pad_get_element_private (pad);
-      if (!stream) {
-        GST_STREAM_SYNCHRONIZER_UNLOCK (self);
-        GST_WARNING_OBJECT (pad, "EOS for unknown stream");
-        break;
+      if (stream) {
+        GST_DEBUG_OBJECT (pad, "Have EOS for stream %d", stream->stream_number);
+        stream->is_eos = TRUE;
       }
 
-      GST_DEBUG_OBJECT (pad, "Have EOS for stream %d", stream->stream_number);
-      stream->is_eos = TRUE;
-
       seen_data = stream->seen_data;
       srcpad = gst_object_ref (stream->srcpad);
 
@@ -546,8 +521,6 @@ gst_stream_synchronizer_sink_event (GstPad * pad, GstEvent * event)
       break;
   }
 
-skip_adjustments:
-
   opad = gst_stream_get_other_pad_from_pad (pad);
   if (opad) {
     ret = gst_pad_push_event (opad, event);
@@ -565,7 +538,7 @@ gst_stream_synchronizer_sink_bufferalloc (GstPad * pad, guint64 offset,
     guint size, GstCaps * caps, GstBuffer ** buf)
 {
   GstPad *opad;
-  GstFlowReturn ret = GST_FLOW_OK;
+  GstFlowReturn ret = GST_FLOW_ERROR;
 
   GST_LOG_OBJECT (pad, "Allocating buffer: size=%u", size);
 
@@ -573,10 +546,6 @@ gst_stream_synchronizer_sink_bufferalloc (GstPad * pad, guint64 offset,
   if (opad) {
     ret = gst_pad_alloc_buffer (opad, offset, size, caps, buf);
     gst_object_unref (opad);
-  } else {
-    /* may have been released during shutdown;
-     * silently trigger fallback */
-    *buf = NULL;
   }
 
   GST_LOG_OBJECT (pad, "Allocation: %s", gst_flow_get_name (ret));
@@ -595,15 +564,6 @@ gst_stream_synchronizer_sink_chain (GstPad * pad, GstBuffer * buffer)
   GstClockTime timestamp = GST_CLOCK_TIME_NONE;
   GstClockTime timestamp_end = GST_CLOCK_TIME_NONE;
 
-  if (passthrough) {
-    opad = gst_stream_get_other_pad_from_pad (pad);
-    if (opad) {
-      ret = gst_pad_push (opad, buffer);
-      gst_object_unref (opad);
-    }
-    goto done;
-  }
-
   GST_LOG_OBJECT (pad, "Handling buffer %p: size=%u, timestamp=%"
       GST_TIME_FORMAT " duration=%" GST_TIME_FORMAT
       " offset=%" G_GUINT64_FORMAT " offset_end=%" G_GUINT64_FORMAT,
@@ -620,8 +580,7 @@ gst_stream_synchronizer_sink_chain (GstPad * pad, GstBuffer * buffer)
   GST_STREAM_SYNCHRONIZER_LOCK (self);
   stream = gst_pad_get_element_private (pad);
 
-  if (stream)
-    stream->seen_data = TRUE;
+  stream->seen_data = TRUE;
   if (stream && stream->drop_discont) {
     buffer = gst_buffer_make_metadata_writable (buffer);
     GST_BUFFER_FLAG_UNSET (buffer, GST_BUFFER_FLAG_DISCONT);
@@ -702,8 +661,6 @@ gst_stream_synchronizer_sink_chain (GstPad * pad, GstBuffer * buffer)
     GST_STREAM_SYNCHRONIZER_UNLOCK (self);
   }
 
-done:
-
   gst_object_unref (self);
 
   return ret;
@@ -971,8 +928,10 @@ gst_stream_synchronizer_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (gstelement_class, &srctemplate);
-  gst_element_class_add_static_pad_template (gstelement_class, &sinktemplate);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&srctemplate));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&sinktemplate));
 
   gst_element_class_set_details_simple (gstelement_class,
       "Stream Synchronizer", "Generic",
