@@ -76,13 +76,6 @@
 GST_DEBUG_CATEGORY_STATIC (video_rate_debug);
 #define GST_CAT_DEFAULT video_rate_debug
 
-/* elementfactory information */
-static const GstElementDetails video_rate_details =
-GST_ELEMENT_DETAILS ("Video rate adjuster",
-    "Filter/Effect/Video",
-    "Drops/duplicates/adjusts timestamps on video frames to make a perfect stream",
-    "Wim Taymans <wim@fluendo.com>");
-
 /* GstVideoRate signals and args */
 enum
 {
@@ -92,6 +85,7 @@ enum
 
 #define DEFAULT_SILENT          TRUE
 #define DEFAULT_NEW_PREF        1.0
+#define DEFAULT_SKIP_TO_FIRST   FALSE
 
 enum
 {
@@ -102,21 +96,24 @@ enum
   ARG_DROP,
   ARG_SILENT,
   ARG_NEW_PREF,
-  /* FILL ME */
+  ARG_SKIP_TO_FIRST
+      /* FILL ME */
 };
 
 static GstStaticPadTemplate gst_video_rate_src_template =
     GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("video/x-raw-yuv; video/x-raw-rgb; image/jpeg; image/png")
+    GST_STATIC_CAPS ("video/x-raw-yuv;"
+        "video/x-raw-rgb;" "video/x-raw-gray;" "image/jpeg;" "image/png")
     );
 
 static GstStaticPadTemplate gst_video_rate_sink_template =
     GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("video/x-raw-yuv; video/x-raw-rgb; image/jpeg; image/png")
+    GST_STATIC_CAPS ("video/x-raw-yuv;"
+        "video/x-raw-rgb;" "video/x-raw-gray;" "image/jpeg;" "image/png")
     );
 
 static void gst_video_rate_swap_prev (GstVideoRate * videorate,
@@ -135,6 +132,9 @@ static GstStateChangeReturn gst_video_rate_change_state (GstElement * element,
 
 /*static guint gst_video_rate_signals[LAST_SIGNAL] = { 0 }; */
 
+static GParamSpec *pspec_drop = NULL;
+static GParamSpec *pspec_duplicate = NULL;
+
 GST_BOILERPLATE (GstVideoRate, gst_video_rate, GstElement, GST_TYPE_ELEMENT);
 
 static void
@@ -142,7 +142,10 @@ gst_video_rate_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_set_details (element_class, &video_rate_details);
+  gst_element_class_set_details_simple (element_class,
+      "Video rate adjuster", "Filter/Effect/Video",
+      "Drops/duplicates/adjusts timestamps on video frames to make a perfect stream",
+      "Wim Taymans <wim@fluendo.com>");
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_video_rate_sink_template));
@@ -168,13 +171,13 @@ gst_video_rate_class_init (GstVideoRateClass * klass)
   g_object_class_install_property (object_class, ARG_OUT,
       g_param_spec_uint64 ("out", "Out", "Number of output frames", 0,
           G_MAXUINT64, 0, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (object_class, ARG_DUP,
-      g_param_spec_uint64 ("duplicate", "Duplicate",
-          "Number of duplicated frames", 0, G_MAXUINT64, 0,
-          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (object_class, ARG_DROP,
-      g_param_spec_uint64 ("drop", "Drop", "Number of dropped frames", 0,
-          G_MAXUINT64, 0, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+  pspec_duplicate = g_param_spec_uint64 ("duplicate", "Duplicate",
+      "Number of duplicated frames", 0, G_MAXUINT64, 0,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, ARG_DUP, pspec_duplicate);
+  pspec_drop = g_param_spec_uint64 ("drop", "Drop", "Number of dropped frames",
+      0, G_MAXUINT64, 0, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, ARG_DROP, pspec_drop);
   g_object_class_install_property (object_class, ARG_SILENT,
       g_param_spec_boolean ("silent", "silent",
           "Don't emit notify for dropped and duplicated frames", DEFAULT_SILENT,
@@ -184,7 +187,19 @@ gst_video_rate_class_init (GstVideoRateClass * klass)
           "Value indicating how much to prefer new frames (unused)", 0.0, 1.0,
           DEFAULT_NEW_PREF, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  element_class->change_state = gst_video_rate_change_state;
+  /**
+   * GstVideoRate:skip-to-first:
+   * 
+   * Don't produce buffers before the first one we receive.
+   *
+   * Since: 0.10.25
+   */
+  g_object_class_install_property (object_class, ARG_SKIP_TO_FIRST,
+      g_param_spec_boolean ("skip-to-first", "Skip to first buffer",
+          "Don't produce buffers before the first one we receive",
+          DEFAULT_SKIP_TO_FIRST, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  element_class->change_state = GST_DEBUG_FUNCPTR (gst_video_rate_change_state);
 }
 
 /* return the caps that can be used on out_pad given in_caps on in_pad */
@@ -195,6 +210,8 @@ gst_video_rate_transformcaps (GstPad * in_pad, GstCaps * in_caps,
   GstCaps *intersect;
   const GstCaps *in_templ;
   gint i;
+  GSList *extra_structures = NULL;
+  GSList *iter;
 
   in_templ = gst_pad_get_pad_template_caps (in_pad);
   intersect = gst_caps_intersect (in_caps, in_templ);
@@ -205,9 +222,22 @@ gst_video_rate_transformcaps (GstPad * in_pad, GstCaps * in_caps,
 
     structure = gst_caps_get_structure (intersect, i);
 
-    gst_structure_set (structure,
-        "framerate", GST_TYPE_FRACTION_RANGE, 0, 1, G_MAXINT, 1, NULL);
+    if (gst_structure_has_field (structure, "framerate")) {
+      GstStructure *copy_structure;
+
+      copy_structure = gst_structure_copy (structure);
+      gst_structure_set (copy_structure,
+          "framerate", GST_TYPE_FRACTION_RANGE, 0, 1, G_MAXINT, 1, NULL);
+      extra_structures = g_slist_append (extra_structures, copy_structure);
+    }
   }
+
+  /* append the extra structures */
+  for (iter = extra_structures; iter != NULL; iter = g_slist_next (iter)) {
+    gst_caps_append_structure (intersect, (GstStructure *) iter->data);
+  }
+  g_slist_free (extra_structures);
+
   *out_caps = intersect;
 
   return TRUE;
@@ -252,7 +282,7 @@ gst_video_rate_setcaps (GstPad * pad, GstCaps * caps)
 
   videorate = GST_VIDEO_RATE (gst_pad_get_parent (pad));
 
-  GST_DEBUG ("setcaps called %" GST_PTR_FORMAT, caps);
+  GST_DEBUG_OBJECT (pad, "setcaps called %" GST_PTR_FORMAT, caps);
 
   structure = gst_caps_get_structure (caps, 0);
   if (!gst_structure_get_fraction (structure, "framerate",
@@ -279,7 +309,6 @@ gst_video_rate_setcaps (GstPad * pad, GstCaps * caps)
       ret = TRUE;
     } else {
       GstCaps *peercaps;
-      GstCaps *intersect;
       GstCaps *transform = NULL;
 
       ret = FALSE;
@@ -291,19 +320,18 @@ gst_video_rate_setcaps (GstPad * pad, GstCaps * caps)
       /* see what the peer can do */
       peercaps = gst_pad_get_caps (opeer);
 
-      GST_DEBUG ("icaps %" GST_PTR_FORMAT, peercaps);
-      GST_DEBUG ("transform %" GST_PTR_FORMAT, transform);
+      GST_DEBUG_OBJECT (opeer, "icaps %" GST_PTR_FORMAT, peercaps);
+      GST_DEBUG_OBJECT (videorate, "transform %" GST_PTR_FORMAT, transform);
 
       /* filter against our possibilities */
-      intersect = gst_caps_intersect (peercaps, transform);
+      caps = gst_caps_intersect (peercaps, transform);
       gst_caps_unref (peercaps);
       gst_caps_unref (transform);
 
-      GST_DEBUG ("intersect %" GST_PTR_FORMAT, intersect);
+      GST_DEBUG_OBJECT (videorate, "intersect %" GST_PTR_FORMAT, caps);
 
       /* take first possibility */
-      caps = gst_caps_copy_nth (intersect, 0);
-      gst_caps_unref (intersect);
+      gst_caps_truncate (caps);
       structure = gst_caps_get_structure (caps, 0);
 
       /* and fixate */
@@ -320,6 +348,17 @@ gst_video_rate_setcaps (GstPad * pad, GstCaps * caps)
         videorate->from_rate_numerator = rate_numerator;
         videorate->from_rate_denominator = rate_denominator;
       }
+
+      if (gst_structure_has_field (structure, "interlaced"))
+        gst_structure_fixate_field_boolean (structure, "interlaced", FALSE);
+      if (gst_structure_has_field (structure, "color-matrix"))
+        gst_structure_fixate_field_string (structure, "color-matrix", "sdtv");
+      if (gst_structure_has_field (structure, "chroma-site"))
+        gst_structure_fixate_field_string (structure, "chroma-site", "mpeg2");
+      if (gst_structure_has_field (structure, "pixel-aspect-ratio"))
+        gst_structure_fixate_field_nearest_fraction (structure,
+            "pixel-aspect-ratio", 1, 1);
+
       gst_pad_set_caps (otherpad, caps);
       gst_caps_unref (caps);
       ret = TRUE;
@@ -329,7 +368,7 @@ gst_video_rate_setcaps (GstPad * pad, GstCaps * caps)
 done:
   /* After a setcaps, our caps may have changed. In that case, we can't use
    * the old buffer, if there was one (it might have different dimensions) */
-  GST_DEBUG ("swapping old buffers");
+  GST_DEBUG_OBJECT (videorate, "swapping old buffers");
   gst_video_rate_swap_prev (videorate, NULL, GST_CLOCK_TIME_NONE);
 
   gst_object_unref (videorate);
@@ -351,7 +390,7 @@ no_transform:
 static void
 gst_video_rate_reset (GstVideoRate * videorate)
 {
-  GST_DEBUG ("resetting internal variables");
+  GST_DEBUG_OBJECT (videorate, "resetting internal variables");
 
   videorate->in = 0;
   videorate->out = 0;
@@ -369,20 +408,26 @@ gst_video_rate_reset (GstVideoRate * videorate)
 static void
 gst_video_rate_init (GstVideoRate * videorate, GstVideoRateClass * klass)
 {
-  GST_DEBUG ("gst_video_rate_init");
   videorate->sinkpad =
       gst_pad_new_from_static_template (&gst_video_rate_sink_template, "sink");
-  gst_pad_set_event_function (videorate->sinkpad, gst_video_rate_event);
-  gst_pad_set_chain_function (videorate->sinkpad, gst_video_rate_chain);
-  gst_pad_set_getcaps_function (videorate->sinkpad, gst_video_rate_getcaps);
-  gst_pad_set_setcaps_function (videorate->sinkpad, gst_video_rate_setcaps);
+  gst_pad_set_event_function (videorate->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_video_rate_event));
+  gst_pad_set_chain_function (videorate->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_video_rate_chain));
+  gst_pad_set_getcaps_function (videorate->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_video_rate_getcaps));
+  gst_pad_set_setcaps_function (videorate->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_video_rate_setcaps));
   gst_element_add_pad (GST_ELEMENT (videorate), videorate->sinkpad);
 
   videorate->srcpad =
       gst_pad_new_from_static_template (&gst_video_rate_src_template, "src");
-  gst_pad_set_query_function (videorate->srcpad, gst_video_rate_query);
-  gst_pad_set_getcaps_function (videorate->srcpad, gst_video_rate_getcaps);
-  gst_pad_set_setcaps_function (videorate->srcpad, gst_video_rate_setcaps);
+  gst_pad_set_query_function (videorate->srcpad,
+      GST_DEBUG_FUNCPTR (gst_video_rate_query));
+  gst_pad_set_getcaps_function (videorate->srcpad,
+      GST_DEBUG_FUNCPTR (gst_video_rate_getcaps));
+  gst_pad_set_setcaps_function (videorate->srcpad,
+      GST_DEBUG_FUNCPTR (gst_video_rate_setcaps));
   gst_element_add_pad (GST_ELEMENT (videorate), videorate->srcpad);
 
   gst_video_rate_reset (videorate);
@@ -397,7 +442,7 @@ gst_video_rate_init (GstVideoRate * videorate, GstVideoRateClass * klass)
 
 /* flush the oldest buffer */
 static GstFlowReturn
-gst_video_rate_flush_prev (GstVideoRate * videorate)
+gst_video_rate_flush_prev (GstVideoRate * videorate, gboolean duplicate)
 {
   GstFlowReturn res;
   GstBuffer *outbuf;
@@ -419,6 +464,11 @@ gst_video_rate_flush_prev (GstVideoRate * videorate)
   } else
     GST_BUFFER_FLAG_UNSET (outbuf, GST_BUFFER_FLAG_DISCONT);
 
+  if (duplicate)
+    GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_GAP);
+  else
+    GST_BUFFER_FLAG_UNSET (outbuf, GST_BUFFER_FLAG_GAP);
+
   /* this is the timestamp we put on the buffer */
   push_ts = videorate->next_ts;
 
@@ -465,6 +515,26 @@ gst_video_rate_swap_prev (GstVideoRate * videorate, GstBuffer * buffer,
   videorate->prev_ts = time;
 }
 
+static void
+gst_video_rate_notify_drop (GstVideoRate * videorate)
+{
+#if !GLIB_CHECK_VERSION(2,26,0)
+  g_object_notify ((GObject *) videorate, "drop");
+#else
+  g_object_notify_by_pspec ((GObject *) videorate, pspec_drop);
+#endif
+}
+
+static void
+gst_video_rate_notify_duplicate (GstVideoRate * videorate)
+{
+#if !GLIB_CHECK_VERSION(2,26,0)
+  g_object_notify ((GObject *) videorate, "duplicate");
+#else
+  g_object_notify_by_pspec ((GObject *) videorate, pspec_duplicate);
+#endif
+}
+
 #define MAGIC_LIMIT  25
 static gboolean
 gst_video_rate_event (GstPad * pad, GstEvent * event)
@@ -504,17 +574,17 @@ gst_video_rate_event (GstPad * pad, GstEvent * event)
                     videorate->next_ts - videorate->segment.accum
                     < videorate->segment.stop)
                 || count < 1)) {
-          gst_video_rate_flush_prev (videorate);
+          res = gst_video_rate_flush_prev (videorate, count > 0);
           count++;
         }
         if (count > 1) {
           videorate->dup += count - 1;
           if (!videorate->silent)
-            g_object_notify (G_OBJECT (videorate), "duplicate");
+            gst_video_rate_notify_duplicate (videorate);
         } else if (count == 0) {
           videorate->drop++;
           if (!videorate->silent)
-            g_object_notify (G_OBJECT (videorate), "drop");
+            gst_video_rate_notify_drop (videorate);
         }
         /* clean up for the new one; _chain will resume from the new start */
         videorate->segment_out = 0;
@@ -530,11 +600,55 @@ gst_video_rate_event (GstPad * pad, GstEvent * event)
           &videorate->segment);
       break;
     }
-    case GST_EVENT_EOS:
-      /* flush last queued frame */
+    case GST_EVENT_EOS:{
+      gint count = 0;
+      GstFlowReturn res = GST_FLOW_OK;
+
       GST_DEBUG_OBJECT (videorate, "Got EOS");
-      gst_video_rate_flush_prev (videorate);
+
+      /* If the segment has a stop position, fill the segment */
+      if (GST_CLOCK_TIME_IS_VALID (videorate->segment.stop)) {
+        /* fill up to the end of current segment,
+         * or only send out the stored buffer if there is no specific stop.
+         * regardless, prevent going loopy in strange cases */
+        while (res == GST_FLOW_OK && count <= MAGIC_LIMIT &&
+            ((videorate->next_ts - videorate->segment.accum <
+                    videorate->segment.stop)
+                || count < 1)) {
+          res = gst_video_rate_flush_prev (videorate, count > 0);
+          count++;
+        }
+      } else if (videorate->prevbuf) {
+        /* Output at least one frame but if the buffer duration is valid, output
+         * enough frames to use the complete buffer duration */
+        if (GST_BUFFER_DURATION_IS_VALID (videorate->prevbuf)) {
+          GstClockTime end_ts =
+              videorate->next_ts + GST_BUFFER_DURATION (videorate->prevbuf);
+
+          while (res == GST_FLOW_OK && count <= MAGIC_LIMIT &&
+              ((videorate->next_ts - videorate->segment.accum < end_ts)
+                  || count < 1)) {
+            res = gst_video_rate_flush_prev (videorate, count > 0);
+            count++;
+          }
+        } else {
+          res = gst_video_rate_flush_prev (videorate, FALSE);
+          count = 1;
+        }
+      }
+
+      if (count > 1) {
+        videorate->dup += count - 1;
+        if (!videorate->silent)
+          gst_video_rate_notify_duplicate (videorate);
+      } else if (count == 0) {
+        videorate->drop++;
+        if (!videorate->silent)
+          gst_video_rate_notify_drop (videorate);
+      }
+
       break;
+    }
     case GST_EVENT_FLUSH_STOP:
       /* also resets the segment */
       GST_DEBUG_OBJECT (videorate, "Got FLUSH_STOP");
@@ -667,7 +781,16 @@ gst_video_rate_chain (GstPad * pad, GstBuffer * buffer)
     if (!GST_CLOCK_TIME_IS_VALID (videorate->next_ts)) {
       /* new buffer, we expect to output a buffer that matches the first
        * timestamp in the segment */
-      videorate->next_ts = videorate->segment.start + videorate->segment.accum;
+      if (videorate->skip_to_first) {
+        videorate->next_ts = in_ts;
+        videorate->segment_out = gst_util_uint64_scale (in_ts,
+            videorate->to_rate_numerator,
+            videorate->to_rate_denominator * GST_SECOND) -
+            (videorate->segment.accum + videorate->segment.start);
+      } else {
+        videorate->next_ts =
+            videorate->segment.start + videorate->segment.accum;
+      }
     }
   } else {
     GstClockTime prevtime;
@@ -692,7 +815,7 @@ gst_video_rate_chain (GstPad * pad, GstBuffer * buffer)
           GST_TIME_ARGS (intime), GST_TIME_ARGS (prevtime));
       videorate->drop++;
       if (!videorate->silent)
-        g_object_notify (G_OBJECT (videorate), "drop");
+        gst_video_rate_notify_drop (videorate);
       gst_buffer_unref (buffer);
       goto done;
     }
@@ -720,7 +843,9 @@ gst_video_rate_chain (GstPad * pad, GstBuffer * buffer)
         count++;
 
         /* on error the _flush function posted a warning already */
-        if ((res = gst_video_rate_flush_prev (videorate)) != GST_FLOW_OK) {
+        if ((res =
+                gst_video_rate_flush_prev (videorate,
+                    count > 1)) != GST_FLOW_OK) {
           gst_buffer_unref (buffer);
           goto done;
         }
@@ -734,14 +859,14 @@ gst_video_rate_chain (GstPad * pad, GstBuffer * buffer)
     if (count > 1) {
       videorate->dup += count - 1;
       if (!videorate->silent)
-        g_object_notify (G_OBJECT (videorate), "duplicate");
+        gst_video_rate_notify_duplicate (videorate);
     }
     /* if we didn't output the first buffer, we have a drop */
     else if (count == 0) {
       videorate->drop++;
 
       if (!videorate->silent)
-        g_object_notify (G_OBJECT (videorate), "drop");
+        gst_video_rate_notify_drop (videorate);
 
       GST_LOG_OBJECT (videorate,
           "new is best, old never used, drop, outgoing ts %"
@@ -750,7 +875,8 @@ gst_video_rate_chain (GstPad * pad, GstBuffer * buffer)
     GST_LOG_OBJECT (videorate,
         "END, putting new in old, diff1 %" GST_TIME_FORMAT
         ", diff2 %" GST_TIME_FORMAT ", next_ts %" GST_TIME_FORMAT
-        ", in %lld, out %lld, drop %lld, dup %lld", GST_TIME_ARGS (diff1),
+        ", in %" G_GUINT64_FORMAT ", out %" G_GUINT64_FORMAT ", drop %"
+        G_GUINT64_FORMAT ", dup %" G_GUINT64_FORMAT, GST_TIME_ARGS (diff1),
         GST_TIME_ARGS (diff2), GST_TIME_ARGS (videorate->next_ts),
         videorate->in, videorate->out, videorate->drop, videorate->dup);
 
@@ -791,6 +917,9 @@ gst_video_rate_set_property (GObject * object,
     case ARG_NEW_PREF:
       videorate->new_pref = g_value_get_double (value);
       break;
+    case ARG_SKIP_TO_FIRST:
+      videorate->skip_to_first = g_value_get_boolean (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -822,6 +951,9 @@ gst_video_rate_get_property (GObject * object,
     case ARG_NEW_PREF:
       g_value_set_double (value, videorate->new_pref);
       break;
+    case ARG_SKIP_TO_FIRST:
+      g_value_set_boolean (value, videorate->skip_to_first);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
