@@ -40,6 +40,14 @@
 GST_DEBUG_CATEGORY_STATIC (seek_debug);
 #define GST_CAT_DEFAULT (seek_debug)
 
+#if !GTK_CHECK_VERSION (2, 17, 7)
+static void
+gtk_widget_get_allocation (GtkWidget * w, GtkAllocation * a)
+{
+  *a = w->allocation;
+}
+#endif
+
 /* configuration */
 
 //#define SOURCE "filesrc"
@@ -57,7 +65,7 @@ GST_DEBUG_CATEGORY_STATIC (seek_debug);
 #define FILL_INTERVAL 100
 //#define UPDATE_INTERVAL 500
 //#define UPDATE_INTERVAL 100
-#define UPDATE_INTERVAL 10
+#define UPDATE_INTERVAL 40
 
 /* number of milliseconds to play for after a seek */
 #define SCRUB_TIME 100
@@ -70,6 +78,7 @@ GST_DEBUG_CATEGORY_STATIC (seek_debug);
 /* the state to go to when stop is pressed */
 #define STOP_STATE      GST_STATE_READY
 
+#define N_GRAD 1000.0
 
 static GList *seekable_pads = NULL;
 static GList *rate_pads = NULL;
@@ -112,7 +121,7 @@ static GtkWidget *video_combo, *audio_combo, *text_combo, *vis_combo;
 static GtkWidget *vis_checkbox, *video_checkbox, *audio_checkbox;
 static GtkWidget *text_checkbox, *mute_checkbox, *volume_spinbutton;
 static GtkWidget *skip_checkbox, *video_window, *download_checkbox;
-static GtkWidget *rate_spinbutton;
+static GtkWidget *buffer_checkbox, *rate_spinbutton;
 
 static GStaticMutex state_mutex = G_STATIC_MUTEX_INIT;
 
@@ -147,7 +156,7 @@ typedef struct
 dyn_link;
 
 static GstElement *
-gst_element_factory_make_or_warn (gchar * type, gchar * name)
+gst_element_factory_make_or_warn (const gchar * type, const gchar * name)
 {
   GstElement *element = gst_element_factory_make (type, name);
 
@@ -965,7 +974,7 @@ make_parselaunch_pipeline (const gchar * description)
 
 typedef struct
 {
-  gchar *name;
+  const gchar *name;
   GstElement *(*func) (const gchar * location);
 }
 Pipeline;
@@ -1004,9 +1013,9 @@ format_value (GtkScale * scale, gdouble value)
   gint64 seconds;
   gint64 subseconds;
 
-  real = value * duration / 100;
+  real = value * duration / N_GRAD;
   seconds = (gint64) real / GST_SECOND;
-  subseconds = (gint64) real / (GST_SECOND / 100);
+  subseconds = (gint64) real / (GST_SECOND / N_GRAD);
 
   return g_strdup_printf ("%02" G_GINT64_FORMAT ":%02" G_GINT64_FORMAT ":%02"
       G_GINT64_FORMAT, seconds / 60, seconds % 60, subseconds % 100);
@@ -1164,22 +1173,32 @@ update_fill (gpointer data)
 
       query = gst_query_new_buffering (GST_FORMAT_PERCENT);
       if (gst_element_query (element, query)) {
-        gint64 start, stop;
+        gint64 start, stop, buffering_total;
         GstFormat format;
         gdouble fill;
         gboolean busy;
         gint percent;
+        GstBufferingMode mode;
+        gint avg_in, avg_out;
+        gint64 buffering_left;
 
         gst_query_parse_buffering_percent (query, &busy, &percent);
-        gst_query_parse_buffering_range (query, &format, &start, &stop, NULL);
+        gst_query_parse_buffering_range (query, &format, &start, &stop,
+            &buffering_total);
+        gst_query_parse_buffering_stats (query, &mode, &avg_in, &avg_out,
+            &buffering_left);
 
+        /* note that we could start the playback when buffering_left < remaining
+         * playback time */
+        GST_DEBUG ("buffering total %" G_GINT64_FORMAT " ms, left %"
+            G_GINT64_FORMAT " ms", buffering_total, buffering_left);
         GST_DEBUG ("start %" G_GINT64_FORMAT ", stop %" G_GINT64_FORMAT,
             start, stop);
 
         if (stop != -1)
-          fill = 100.0 * stop / GST_FORMAT_PERCENT_MAX;
+          fill = N_GRAD * stop / GST_FORMAT_PERCENT_MAX;
         else
-          fill = 100.0;
+          fill = N_GRAD;
 
         gtk_range_set_fill_level (GTK_RANGE (hscale), fill);
       }
@@ -1226,7 +1245,7 @@ update_scale (gpointer data)
     duration = position;
 
   if (duration > 0) {
-    set_scale (position * 100.0 / duration);
+    set_scale (position * N_GRAD / duration);
   }
 
   /* FIXME: see make_playerbin2_pipeline() and volume_notify_cb() */
@@ -1296,7 +1315,10 @@ do_seek (GtkWidget * widget)
   GstEvent *s_event;
   GstSeekFlags flags;
 
-  real = gtk_range_get_value (GTK_RANGE (widget)) * duration / 100;
+  real = gtk_range_get_value (GTK_RANGE (widget)) * duration / N_GRAD;
+
+  GST_DEBUG ("value=%f, real=%" G_GINT64_FORMAT,
+      gtk_range_get_value (GTK_RANGE (widget)), real);
 
   flags = 0;
   if (flush_seek)
@@ -1412,8 +1434,9 @@ start_seek (GtkWidget * widget, GdkEventButton * event, gpointer user_data)
   }
 
   if (changed_id == 0 && flush_seek && scrub) {
-    changed_id = g_signal_connect (GTK_OBJECT (hscale),
-        "value_changed", G_CALLBACK (seek_cb), pipeline);
+    changed_id =
+        g_signal_connect (hscale, "value_changed", G_CALLBACK (seek_cb),
+        pipeline);
   }
 
   return FALSE;
@@ -1423,7 +1446,7 @@ static gboolean
 stop_seek (GtkWidget * widget, GdkEventButton * event, gpointer user_data)
 {
   if (changed_id) {
-    g_signal_handler_disconnect (GTK_OBJECT (hscale), changed_id);
+    g_signal_handler_disconnect (hscale, changed_id);
     changed_id = 0;
   }
 
@@ -1739,6 +1762,15 @@ download_toggle_cb (GtkToggleButton * button, GstPipeline * pipeline)
   update_flag (pipeline, 7, state);
 }
 
+static void
+buffer_toggle_cb (GtkToggleButton * button, GstPipeline * pipeline)
+{
+  gboolean state;
+
+  state = gtk_toggle_button_get_active (button);
+  update_flag (pipeline, 8, state);
+}
+
 static void
 clear_streams (GstElement * pipeline)
 {
@@ -2374,15 +2406,21 @@ static void
 msg_clock_lost (GstBus * bus, GstMessage * message, GstPipeline * data)
 {
   g_print ("clock lost! PAUSE and PLAY to select a new clock\n");
-
-  gst_element_set_state (pipeline, GST_STATE_PAUSED);
-  gst_element_set_state (pipeline, GST_STATE_PLAYING);
+  if (state == GST_STATE_PLAYING) {
+    gst_element_set_state (pipeline, GST_STATE_PAUSED);
+    gst_element_set_state (pipeline, GST_STATE_PLAYING);
+  }
 }
 
 #ifdef HAVE_X
 
-static guint embed_xid = 0;
+static gulong embed_xid = 0;
 
+/* We set the xid here in response to the prepare-xwindow-id message via a
+ * bus sync handler because we don't know the actual videosink used from the
+ * start (as we don't know the pipeline, or bin elements such as autovideosink
+ * or gconfvideosink may be used which create the actual videosink only once
+ * the pipeline is started) */
 static GstBusSyncReply
 bus_sync_handler (GstBus * bus, GstMessage * message, GstPipeline * data)
 {
@@ -2390,18 +2428,21 @@ bus_sync_handler (GstBus * bus, GstMessage * message, GstPipeline * data)
       gst_structure_has_name (message->structure, "prepare-xwindow-id")) {
     GstElement *element = GST_ELEMENT (GST_MESSAGE_SRC (message));
 
-    g_print ("got prepare-xwindow-id\n");
-    if (!embed_xid) {
-      embed_xid = GDK_WINDOW_XID (GDK_WINDOW (video_window->window));
-    }
+    g_print ("got prepare-xwindow-id, setting XID %lu\n", embed_xid);
 
     if (g_object_class_find_property (G_OBJECT_GET_CLASS (element),
             "force-aspect-ratio")) {
       g_object_set (element, "force-aspect-ratio", TRUE, NULL);
     }
 
-    gst_x_overlay_set_xwindow_id (GST_X_OVERLAY (GST_MESSAGE_SRC (message)),
-        embed_xid);
+    /* Should have been initialised from main thread before (can't use
+     * GDK_WINDOW_XID here with Gtk+ >= 2.18, because the sync handler will
+     * be called from a streaming thread and GDK_WINDOW_XID maps to more than
+     * a simple structure lookup with Gtk+ >= 2.18, where 'more' is stuff that
+     * shouldn't be done from a non-GUI thread without explicit locking).  */
+    g_assert (embed_xid != 0);
+
+    gst_x_overlay_set_window_handle (GST_X_OVERLAY (element), embed_xid);
   }
   return GST_BUS_PASS;
 }
@@ -2411,12 +2452,43 @@ static gboolean
 handle_expose_cb (GtkWidget * widget, GdkEventExpose * event, gpointer data)
 {
   if (state < GST_STATE_PAUSED) {
-    gdk_draw_rectangle (widget->window, widget->style->black_gc, TRUE,
-        0, 0, widget->allocation.width, widget->allocation.height);
+    GtkAllocation allocation;
+    GdkWindow *window = gtk_widget_get_window (widget);
+    cairo_t *cr;
+
+    gtk_widget_get_allocation (widget, &allocation);
+    cr = gdk_cairo_create (window);
+    cairo_set_source_rgb (cr, 0, 0, 0);
+    cairo_rectangle (cr, 0, 0, allocation.width, allocation.height);
+    cairo_fill (cr);
+    cairo_destroy (cr);
   }
   return FALSE;
 }
 
+static void
+realize_cb (GtkWidget * widget, gpointer data)
+{
+#if GTK_CHECK_VERSION(2,18,0)
+  {
+    GdkWindow *window = gtk_widget_get_window (widget);
+
+    /* This is here just for pedagogical purposes, GDK_WINDOW_XID will call it
+     * as well */
+    if (!gdk_window_ensure_native (window))
+      g_error ("Couldn't create native window needed for GstXOverlay!");
+  }
+#endif
+
+#ifdef HAVE_X
+  {
+    GdkWindow *window = gtk_widget_get_window (video_window);
+
+    embed_xid = GDK_WINDOW_XID (window);
+    g_print ("Window realize: video window XID = %lu\n", embed_xid);
+  }
+#endif
+}
 
 static void
 msg_eos (GstBus * bus, GstMessage * message, GstPipeline * data)
@@ -2593,8 +2665,9 @@ main (int argc, char **argv)
 
   pipeline_spec = argv[2];
 
-  if (g_strrstr (pipeline_spec, "*") != NULL ||
-      g_strrstr (pipeline_spec, "?") != NULL) {
+  if (g_path_is_absolute (pipeline_spec) &&
+      (g_strrstr (pipeline_spec, "*") != NULL ||
+          g_strrstr (pipeline_spec, "?") != NULL)) {
     paths = handle_wildcards (pipeline_spec);
   } else {
     paths = g_list_prepend (paths, g_strdup (pipeline_spec));
@@ -2613,9 +2686,11 @@ main (int argc, char **argv)
   /* initialize gui elements ... */
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   video_window = gtk_drawing_area_new ();
-  g_signal_connect (G_OBJECT (video_window), "expose-event",
+  g_signal_connect (video_window, "expose-event",
       G_CALLBACK (handle_expose_cb), NULL);
+  g_signal_connect (video_window, "realize", G_CALLBACK (realize_cb), NULL);
   gtk_widget_set_double_buffered (video_window, FALSE);
+
   statusbar = gtk_statusbar_new ();
   status_id = gtk_statusbar_get_context_id (GTK_STATUSBAR (statusbar), "seek");
   gtk_statusbar_push (GTK_STATUSBAR (statusbar), status_id, "Stopped");
@@ -2694,8 +2769,8 @@ main (int argc, char **argv)
     shuttle_checkbox = gtk_check_button_new_with_label ("Shuttle");
     gtk_box_pack_start (GTK_BOX (hbox), shuttle_checkbox, FALSE, FALSE, 2);
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (shuttle_checkbox), FALSE);
-    g_signal_connect (GTK_OBJECT (shuttle_checkbox),
-        "toggled", G_CALLBACK (shuttle_toggled), pipeline);
+    g_signal_connect (shuttle_checkbox, "toggled", G_CALLBACK (shuttle_toggled),
+        pipeline);
 
     shuttle_adjustment =
         GTK_ADJUSTMENT (gtk_adjustment_new (0.0, -3.00, 4.0, 0.1, 1.0, 1.0));
@@ -2704,9 +2779,9 @@ main (int argc, char **argv)
     gtk_scale_set_value_pos (GTK_SCALE (shuttle_hscale), GTK_POS_TOP);
     gtk_range_set_update_policy (GTK_RANGE (shuttle_hscale),
         GTK_UPDATE_CONTINUOUS);
-    g_signal_connect (GTK_OBJECT (shuttle_hscale), "value_changed",
+    g_signal_connect (shuttle_hscale, "value_changed",
         G_CALLBACK (shuttle_value_changed), pipeline);
-    g_signal_connect (GTK_OBJECT (shuttle_hscale), "format_value",
+    g_signal_connect (shuttle_hscale, "format_value",
         G_CALLBACK (shuttle_format_value), pipeline);
 
     gtk_box_pack_start (GTK_BOX (hbox), shuttle_hscale, TRUE, TRUE, 2);
@@ -2716,22 +2791,20 @@ main (int argc, char **argv)
 
   /* seek bar */
   adjustment =
-      GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.00, 100.0, 0.1, 1.0, 1.0));
+      GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.00, N_GRAD, 0.1, 1.0, 1.0));
   hscale = gtk_hscale_new (adjustment);
   gtk_scale_set_digits (GTK_SCALE (hscale), 2);
   gtk_scale_set_value_pos (GTK_SCALE (hscale), GTK_POS_RIGHT);
-#if GTK_CHECK_VERSION(2,12,0)
   gtk_range_set_show_fill_level (GTK_RANGE (hscale), TRUE);
-  gtk_range_set_fill_level (GTK_RANGE (hscale), 100.0);
-#endif
+  gtk_range_set_fill_level (GTK_RANGE (hscale), N_GRAD);
   gtk_range_set_update_policy (GTK_RANGE (hscale), GTK_UPDATE_CONTINUOUS);
 
-  g_signal_connect (GTK_OBJECT (hscale),
-      "button_press_event", G_CALLBACK (start_seek), pipeline);
-  g_signal_connect (GTK_OBJECT (hscale),
-      "button_release_event", G_CALLBACK (stop_seek), pipeline);
-  g_signal_connect (GTK_OBJECT (hscale),
-      "format_value", G_CALLBACK (format_value), pipeline);
+  g_signal_connect (hscale, "button_press_event", G_CALLBACK (start_seek),
+      pipeline);
+  g_signal_connect (hscale, "button_release_event", G_CALLBACK (stop_seek),
+      pipeline);
+  g_signal_connect (hscale, "format_value", G_CALLBACK (format_value),
+      pipeline);
 
   if (pipeline_type == 16) {
     /* the playbin2 panel controls for the video/audio/subtitle tracks */
@@ -2759,6 +2832,7 @@ main (int argc, char **argv)
     text_checkbox = gtk_check_button_new_with_label ("Text");
     mute_checkbox = gtk_check_button_new_with_label ("Mute");
     download_checkbox = gtk_check_button_new_with_label ("Download");
+    buffer_checkbox = gtk_check_button_new_with_label ("Buffer");
     volume_label = gtk_label_new ("Volume");
     volume_spinbutton = gtk_spin_button_new_with_range (0, 10.0, 0.1);
     gtk_spin_button_set_value (GTK_SPIN_BUTTON (volume_spinbutton), 1.0);
@@ -2768,6 +2842,7 @@ main (int argc, char **argv)
     gtk_box_pack_start (GTK_BOX (boxes), vis_checkbox, TRUE, TRUE, 2);
     gtk_box_pack_start (GTK_BOX (boxes), mute_checkbox, TRUE, TRUE, 2);
     gtk_box_pack_start (GTK_BOX (boxes), download_checkbox, TRUE, TRUE, 2);
+    gtk_box_pack_start (GTK_BOX (boxes), buffer_checkbox, TRUE, TRUE, 2);
     gtk_box_pack_start (GTK_BOX (boxes), volume_label, TRUE, TRUE, 2);
     gtk_box_pack_start (GTK_BOX (boxes), volume_spinbutton, TRUE, TRUE, 2);
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (vis_checkbox), FALSE);
@@ -2776,6 +2851,7 @@ main (int argc, char **argv)
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (text_checkbox), TRUE);
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (mute_checkbox), FALSE);
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (download_checkbox), FALSE);
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (buffer_checkbox), FALSE);
     g_signal_connect (G_OBJECT (vis_checkbox), "toggled",
         G_CALLBACK (vis_toggle_cb), pipeline);
     g_signal_connect (G_OBJECT (audio_checkbox), "toggled",
@@ -2788,6 +2864,8 @@ main (int argc, char **argv)
         G_CALLBACK (mute_toggle_cb), pipeline);
     g_signal_connect (G_OBJECT (download_checkbox), "toggled",
         G_CALLBACK (download_toggle_cb), pipeline);
+    g_signal_connect (G_OBJECT (buffer_checkbox), "toggled",
+        G_CALLBACK (buffer_toggle_cb), pipeline);
     g_signal_connect (G_OBJECT (volume_spinbutton), "value_changed",
         G_CALLBACK (volume_spinbutton_changed_cb), pipeline);
     /* playbin2 panel for snapshot */
@@ -2876,6 +2954,16 @@ main (int argc, char **argv)
   /* show the gui. */
   gtk_widget_show_all (window);
 
+  /* realize window now so that the video window gets created and we can
+   * obtain its XID before the pipeline is started up and the videosink
+   * asks for the XID of the window to render onto */
+  gtk_widget_realize (window);
+
+#ifdef HAVE_X
+  /* we should have the XID now */
+  g_assert (embed_xid != 0);
+#endif
+
   if (verbose) {
     g_signal_connect (pipeline, "deep_notify",
         G_CALLBACK (gst_object_default_deep_notify), NULL);
