@@ -34,28 +34,47 @@
 #include <gst/interfaces/xoverlay.h>
 #include <gst/interfaces/propertyprobe.h>
 
+#if !GTK_CHECK_VERSION (2, 17, 7)
+static void
+gtk_widget_get_allocation (GtkWidget * w, GtkAllocation * a)
+{
+  *a = w->allocation;
+}
+#endif
+
 static GtkWidget *video_window = NULL;
 static GstElement *sink = NULL;
 static gulong embed_xid = 0;
-static GdkGC *trans_gc = NULL;
+static GdkColor trans_color;
+static gboolean trans_color_set = FALSE;
 
 static void
 redraw_overlay (GtkWidget * widget)
 {
-  gdk_draw_rectangle (widget->window, widget->style->white_gc, TRUE,
-      0, 0, widget->allocation.width, widget->allocation.height);
+  GtkAllocation allocation;
+  GdkWindow *window = gtk_widget_get_window (widget);
+  cairo_t *cr;
 
-  if (trans_gc) {
+  cr = gdk_cairo_create (window);
+  gtk_widget_get_allocation (widget, &allocation);
+
+  cairo_set_source_rgb (cr, 1, 1, 1);
+  cairo_rectangle (cr, 0, 0, allocation.width, allocation.height);
+  cairo_fill (cr);
+
+  if (trans_color_set) {
     guint x, y;
-    guint h = widget->allocation.height * 0.75;
+    guint h = allocation.height * 0.75;
 
-    gdk_draw_rectangle (widget->window, trans_gc, TRUE,
-        0, 0, widget->allocation.width, h);
+    gdk_cairo_set_source_color (cr, &trans_color);
+    cairo_rectangle (cr, 0, 0, allocation.width, h);
+    cairo_fill (cr);
 
-    for (y = h; y < widget->allocation.height; y++) {
-      for (x = 0; x < widget->allocation.width; x++) {
+    for (y = h; y < allocation.height; y++) {
+      for (x = 0; x < allocation.width; x++) {
         if (((x & 1) || (y & 1)) && (x & 1) != (y & 1)) {
-          gdk_draw_point (widget->window, trans_gc, x, y);
+          cairo_move_to (cr, x, y);
+          cairo_paint (cr);
         }
       }
     }
@@ -80,14 +99,22 @@ static void
 realize_cb (GtkWidget * widget, gpointer data)
 {
 #if GTK_CHECK_VERSION(2,18,0)
-  /* This is here just for pedagogical purposes, GDK_WINDOW_XID will call it
-   * as well */
-  if (!gdk_window_ensure_native (widget->window))
-    g_error ("Couldn't create native window needed for GstXOverlay!");
+  {
+    GdkWindow *window = gtk_widget_get_window (widget);
+
+    /* This is here just for pedagogical purposes, GDK_WINDOW_XID will call it
+     * as well */
+    if (!gdk_window_ensure_native (window))
+      g_error ("Couldn't create native window needed for GstXOverlay!");
+  }
 #endif
 
-  embed_xid = GDK_WINDOW_XID (video_window->window);
-  g_print ("Window realize: got XID %lu\n", embed_xid);
+  {
+    GdkWindow *window = gtk_widget_get_window (video_window);
+
+    embed_xid = GDK_WINDOW_XID (window);
+    g_print ("Window realize: video window XID = %lu\n", embed_xid);
+  }
 }
 
 static void
@@ -106,20 +133,19 @@ msg_state_changed (GstBus * bus, GstMessage * message, GstPipeline * pipeline)
 
     /* When state of the pipeline changes to paused or playing we start updating scale */
     switch (GST_STATE_TRANSITION (old, new)) {
-      case GST_STATE_CHANGE_READY_TO_PAUSED:
+      case GST_STATE_CHANGE_READY_TO_PAUSED:{
         g_object_get (G_OBJECT (sink), "colorkey", &color, NULL);
         if (color != -1) {
-          GdkColor trans_color = { 0,
-            (color & 0xff0000) >> 8,
-            (color & 0xff00),
-            (color & 0xff) << 8
-          };
-
-          trans_gc = gdk_gc_new (video_window->window);
-          gdk_gc_set_rgb_fg_color (trans_gc, &trans_color);
+          trans_color.red = (color & 0xff0000) >> 8;
+          trans_color.green = (color & 0xff00);
+          trans_color.blue = (color & 0xff) << 8;
+          trans_color_set = TRUE;
+        } else {
+          trans_color_set = FALSE;
         }
         handle_resize_cb (video_window, NULL, NULL);
         break;
+      }
       default:
         break;
     }
@@ -247,7 +273,7 @@ main (int argc, char **argv)
    * just set it directly here now (instead of waiting for a prepare-xwindow-id
    * element message in a sync bus handler and setting it there) */
   g_print ("setting XID %lu\n", embed_xid);
-  gst_x_overlay_set_xwindow_id (GST_X_OVERLAY (sink), embed_xid);
+  gst_x_overlay_set_window_handle (GST_X_OVERLAY (sink), embed_xid);
 
   g_idle_add (start_pipeline, pipeline);
   gtk_main ();
